@@ -29,8 +29,8 @@
 
 #include <poll.h>
 #include <signal.h>
-#include <stdio.h>
-#include <string.h>
+#include <cstdio>
+#include <cstring>
 #include <sys/types.h>
 #include <sys/uio.h>
 
@@ -295,6 +295,16 @@ int XrdLink::Recv(char *Buff, int Blen, int timeout)
    else       return linkXQ.Recv    (Buff, Blen, timeout);
 }
 
+/******************************************************************************/
+
+int XrdLink::Recv(const struct iovec *iov, int iocnt, int timeout)
+{
+// Execute the send
+//
+   if (isTLS) return linkXQ.TLS_Recv(iov, iocnt, timeout);
+   else       return linkXQ.Recv    (iov, iocnt, timeout);
+}
+
 /******************************************************************************/
 /*                               R e c v A l l                                */
 /******************************************************************************/
@@ -363,7 +373,7 @@ void XrdLink::Serialize()
 {
 
 // This is meant to make sure that no protocol objects are refering to this
-// link so that we can safely run in psuedo single thread mode for critical
+// link so that we can safely run in pseudo single thread mode for critical
 // functions.
 //
    linkXQ.LinkInfo.opMutex.Lock();
@@ -445,7 +455,8 @@ void XrdLink::setProtName(const char *name)
 void XrdLink::setRef(int use)
 {
    linkXQ.LinkInfo.opMutex.Lock();
-   TRACEI(DEBUG,"Setting ref to " <<linkXQ.LinkInfo.InUse <<'+' 
+   TRACEI(DEBUG,"Setting FD "<<linkXQ.LinkInfo.FD <<" ref to " 
+                <<linkXQ.LinkInfo.InUse <<'+'
                  <<use <<" post=" <<linkXQ.LinkInfo.doPost);
    linkXQ.LinkInfo.InUse += use;
 
