@@ -85,41 +85,41 @@ const char   *XrdCpConfig::opLetters = ":C:d:D:EfFhHI:NpPrRsS:t:T:vVX:y:z:ZA";
 
 struct option XrdCpConfig::opVec[] =         // For getopt_long()
      {
+      {OPT_TYPE "allow-http",     0, 0, XrdCpConfig::OpAllowHttp},
       {OPT_TYPE "cksum",          1, 0, XrdCpConfig::OpCksum},
+      {OPT_TYPE "coerce",         0, 0, XrdCpConfig::OpCoerce},
+      {OPT_TYPE "continue",       0, 0, XrdCpConfig::OpContinue},
       {OPT_TYPE "debug",          1, 0, XrdCpConfig::OpDebug},
       {OPT_TYPE "dynamic-src",    0, 0, XrdCpConfig::OpDynaSrc},
-      {OPT_TYPE "coerce",         0, 0, XrdCpConfig::OpCoerce},
       {OPT_TYPE "force",          0, 0, XrdCpConfig::OpForce},
       {OPT_TYPE "help",           0, 0, XrdCpConfig::OpHelp},
       {OPT_TYPE "infiles",        1, 0, XrdCpConfig::OpIfile},
       {OPT_TYPE "license",        0, 0, XrdCpConfig::OpLicense},
       {OPT_TYPE "nopbar",         0, 0, XrdCpConfig::OpNoPbar},
       {OPT_TYPE "notlsok",        0, 0, XrdCpConfig::OpNoTlsOK},
+      {OPT_TYPE "parallel",       1, 0, XrdCpConfig::OpParallel},
       {OPT_TYPE "path",           0, 0, XrdCpConfig::OpPath},
       {OPT_TYPE "posc",           0, 0, XrdCpConfig::OpPosc},
       {OPT_TYPE "proxy",          1, 0, XrdCpConfig::OpProxy},
       {OPT_TYPE "recursive",      0, 0, XrdCpConfig::OpRecurse},
       {OPT_TYPE "retry",          1, 0, XrdCpConfig::OpRetry},
+      {OPT_TYPE "retry-policy",   1, 0, XrdCpConfig::OpRetryPolicy},
+      {OPT_TYPE "rm-bad-cksum",   0, 0, XrdCpConfig::OpRmOnBadCksum},
       {OPT_TYPE "server",         0, 0, XrdCpConfig::OpServer},
       {OPT_TYPE "silent",         0, 0, XrdCpConfig::OpSilent},
       {OPT_TYPE "sources",        1, 0, XrdCpConfig::OpSources},
       {OPT_TYPE "streams",        1, 0, XrdCpConfig::OpStreams},
-      {OPT_TYPE "tlsnodata",      0, 0, XrdCpConfig::OpTlsNoData},
       {OPT_TYPE "tlsmetalink",    0, 0, XrdCpConfig::OpTlsMLF},
+      {OPT_TYPE "tlsnodata",      0, 0, XrdCpConfig::OpTlsNoData},
       {OPT_TYPE "tpc",            1, 0, XrdCpConfig::OpTpc},
       {OPT_TYPE "verbose",        0, 0, XrdCpConfig::OpVerbose},
       {OPT_TYPE "version",        0, 0, XrdCpConfig::OpVersion},
-      {OPT_TYPE "xrate",          1, 0, XrdCpConfig::OpXrate},
-      {OPT_TYPE "parallel",       1, 0, XrdCpConfig::OpParallel},
-      {OPT_TYPE "zip",            1, 0, XrdCpConfig::OpZip},
-      {OPT_TYPE "allow-http",     0, 0, XrdCpConfig::OpAllowHttp},
       {OPT_TYPE "xattr",          0, 0, XrdCpConfig::OpXAttr},
-      {OPT_TYPE "zip-mtln-cksum", 0, 0, XrdCpConfig::OpZipMtlnCksum},
-      {OPT_TYPE "rm-bad-cksum",   0, 0, XrdCpConfig::OpRmOnBadCksum},
-      {OPT_TYPE "continue",       0, 0, XrdCpConfig::OpContinue},
-      {OPT_TYPE "xrate-threshold",1, 0, XrdCpConfig::OpXrateThreashold},
-      {OPT_TYPE "retry-policy",   1, 0, XrdCpConfig::OpRetryPolicy},
+      {OPT_TYPE "xrate",          1, 0, XrdCpConfig::OpXrate},
+      {OPT_TYPE "xrate-threshold",1, 0, XrdCpConfig::OpXrateThreshold},
+      {OPT_TYPE "zip",            1, 0, XrdCpConfig::OpZip},
       {OPT_TYPE "zip-append",     0, 0, XrdCpConfig::OpZipAppend},
+      {OPT_TYPE "zip-mtln-cksum", 0, 0, XrdCpConfig::OpZipMtlnCksum},
       {0,                         0, 0, 0}
      };
 
@@ -141,7 +141,7 @@ XrdCpConfig::XrdCpConfig(const char *pgm)
    pHost    = 0;
    pPort    = 0;
    xRate    = 0;
-   xRateThreashold = 0;
+   xRateThreshold = 0;
    Parallel = 1;
    OpSpec   = 0;
    Dlvl     = 0;
@@ -176,6 +176,7 @@ XrdCpConfig::~XrdCpConfig()
    if (inFile)  free(inFile);
    if (pHost)   free(pHost);
    if (parmVal) free(parmVal);
+   if (CksObj)  delete CksObj;
    if (CksMan)  delete CksMan;
    if (zipFile) free(zipFile);
    if (dstFile) delete dstFile;
@@ -313,11 +314,11 @@ do{while(optind < Argc && Legacy(optind)) {}
           case OpXrate:         OpSpec |= DoXrate;
                                 if (!a2z(optarg, &xRate, 10*1024LL, -1)) Usage(22);
                                 break;
-          case OpXrateThreashold: OpSpec |= DoXrateThreashold;
-                                  if (!a2z(optarg, &xRateThreashold, 10*1024LL, -1)) Usage(22);
+          case OpXrateThreshold: OpSpec |= DoXrateThreshold;
+                                  if (!a2z(optarg, &xRateThreshold, 10*1024LL, -1)) Usage(22);
                                   break;
           case OpParallel:      OpSpec |= DoParallel;
-                                if (!a2i(optarg, &Parallel, 1, 4)) Usage(22);
+                                if (!a2i(optarg, &Parallel, 1, 128)) Usage(22);
                                 break;
           case OpAllowHttp:     OpSpec |= DoAllowHttp;
                                 break;
@@ -929,15 +930,16 @@ void XrdCpConfig::Usage(int rc)
    "Usage:   xrdcp [<options>] <src> <dest>\n";
 
    static const char *Options= "\n"
-   "Options: [--cksum <args>] [--debug <lvl>] [--coerce] [--dynamic-src]\n"
-   "         [--force] [--help] [--infiles <fn>] [--license] [--nopbar]\n"
-   "         [--notlsok] [--parallel <n>] [--posc] [--proxy <host>:<port>]\n"
-   "         [--recursive] [--retry <n>] [--server] [--silent] [--sources <n>]\n"
-   "         [--streams <n>] [--tlsnodata] [--tlsmetalink]\n"
+   "Options: [--allow-http] [--cksum <args>] [--coerce] [--continue]\n"
+   "         [--debug <lvl>] [--dynamic-src] [--force] [--help]\n"
+   "         [--infiles <fn>] [--license] [--nopbar] [--notlsok]\n"
+   "         [--parallel <n>] [--posc] [--proxy <host>:<port>]\n"
+   "         [--recursive] [--retry <n>] [--retry-policy <force|continue>]\n"
+   "         [--rm-bad-cksum] [--server] [--silent] [--sources <n>]\n"
+   "         [--streams <n>] [--tlsmetalink] [--tlsnodata]\n"
    "         [--tpc [delegate] {first|only}] [--verbose] [--version]\n"
-   "         [--xrate <rate>] [--zip <file>] [--allow-http] [--xattr]\n"
-   "         [--zip-mtln-cksum] [--rm-bad-cksum] [--continue]\n"
-   "         [--xrate-threshold <rate>] [--retry-policy <force|continue>]\n";
+   "         [--xattr] [--xrate <rate>] [--xrate-threshold <rate>]\n"
+   "         [--zip <file>] [--zip-append] [--zip-mtln-cksum]\n";
 
    static const char *Syntax2= "\n"
    "<src>:   [[x]root[s]://<host>[:<port>]/]<path> | -";
@@ -949,62 +951,66 @@ void XrdCpConfig::Usage(int rc)
    "<dest>:  [[x]root[s]://<host>[:<port>]/]<path> | -";
 
    static const char *Detail = "\n"
+   "Note: using a dash (-) for <src> uses stdin and for <dest> stdout\n\n"
+   "-A | --allow-http             allow HTTP as source or destination protocol. Requires\n"
+   "                              the XrdClHttp client plugin\n"
    "-C | --cksum <args>           verifies the checksum at the destination as provided\n"
    "                              by the source server or locally computed. The args are\n"
-   "                              {adler32 | crc32 | md5 | zcrc32 | auto}[:{<value>|print|source}]\n"
-   "                              If 'auto' is chosen as the checksum type, xrdcp will try to\n"
-   "                              automatically infer the right checksum type based on source/\n"
-   "                              destination configuration, source file type (metalink, ZIP), and \n"
-   "                              available checksum plug-ins.\n"
+   "                                      <ckstype>[:{<value>|print|source}]\n"
+   "                              where <ckstype> is one of adler32, crc32, crc32c, md5,\n"
+   "                              zcrc32 or auto. If 'auto' is chosen, xrdcp will try to\n"
+   "                              automatically infer the right checksum type based on the\n"
+   "                              source/destination configuration, source file type\n"
+   "                              (e.g. metalink, ZIP), and available checksum plug-ins.\n"
    "                              If the hex value of the checksum is given, it is used.\n"
    "                              Otherwise, the server's checksum is used for remote files\n"
    "                              and computed for local files. Specifying print merely\n"
    "                              prints the checksum but does not verify it.\n"
+   "-F | --coerce                 coerces the copy by ignoring file locking semantics\n"
+   "     --continue               continue copying a file from the point where the previous\n"
+   "                              copy was interrupted\n"
    "-d | --debug <lvl>            sets the debug level: 0 off, 1 low, 2 medium, 3 high\n"
    "-Z | --dynamic-src            file size may change during the copy\n"
-   "-F | --coerce                 coerces the copy by ignoring file locking semantics\n"
    "-f | --force                  replaces any existing output file\n"
    "-h | --help                   prints this information\n"
+   "-I | --infiles <fname>        specifies the file that contains a list of input files\n"
    "-H | --license                prints license terms and conditions\n"
-   "-I | --infiles                specifies the file that contains a list of input files\n"
    "-N | --nopbar                 does not print the progress bar\n"
    "     --notlsok                if server is too old to support TLS encryption fallback\n"
    "                              to unencrypted communication\n"
+   "     --parallel <n>           number of files to copy at the same time\n"
    "-P | --posc                   enables persist on successful close semantics\n"
-   "-D | --proxy                  uses the specified SOCKS4 proxy connection\n"
+   "-D | --proxy <host>:<port>    uses the specified SOCKS4 proxy connection\n"
    "-r | --recursive              recursively copies all source files\n"
+   "-t | --retry <n>              maximum number of times to retry failed copy-jobs\n"
+   "     --retry-policy <policy>  retry policy: force or continue\n"
    "     --rm-bad-cksum           remove the target file if checksum verification failed\n"
    "                              (enables also POSC semantics)\n"
-   "-t | --retry <n>              maximum number of times to retry failed copy-jobs\n"
    "     --server                 runs in a server environment with added operations\n"
    "-s | --silent                 produces no output other than error messages\n"
    "-y | --sources <n>            uses up to the number of sources specified in parallel\n"
    "-S | --streams <n>            copies using the specified number of TCP connections\n"
+   "     --tlsmetalink            convert [x]root to [x]roots protocol in metalinks\n"
    "-E | --tlsnodata              in case of [x]roots protocol, encrypt only the control\n"
    "                              stream and leave the data streams unencrypted\n"
-   "     --tlsmetalink            convert [x]root to [x]roots protocol in metalinks\n"
-   "-T | --tpc                    uses third party copy mode between the src and dest.\n"
+   "-T | --tpc <args>             uses third party copy mode between the src and dest.\n"
    "                              Both the src and dest must allow tpc mode. Argument\n"
    "                              'first' tries tpc and if it fails, does a normal copy;\n"
    "                              while 'only' fails the copy unless tpc succeeds.\n"
    "-v | --verbose                produces more information about the copy\n"
    "-V | --version                prints the version number\n"
+   "     --xattr                  preserve extended attributes\n"
    "-X | --xrate <rate>           limits the transfer to the specified rate. You can\n"
    "                              suffix the value with 'k', 'm', or 'g'\n"
    "     --xrate-threshold <rate> If the transfer rate drops below given threshold force\n"
    "                              the client to use different source or if no more sources\n"
    "                              are available fail the transfer. You can suffix the value\n"
    "                              with 'k', 'm', or 'g'\n"
-   "     --parallel <n>           number of copy jobs to be run simultaneously\n\n"
    "-z | --zip <file>             treat the source as a ZIP archive containing given file\n"
-   "-A | --allow-http             allow HTTP as source or destination protocol. Requires\n"
-   "                              the XrdClHttp client plugin\n"
-   "     --xattr                  preserve extended attributes\n"
+   "     --zip-append             append file to existing zip archive\n"
    "     --zip-mtln-cksum         use the checksum available in a metalink file even if\n"
    "                              a file is being extracted from a ZIP archive\n"
-   "     --continue               continue copying a file from the point where the previous\n"
-   "                              copy was interrupted\n"
-   "     --retry-policy           retry policy: force or continue"
+   "\n"
    "Legacy options:     [-adler] [-DI<var> <val>] [-DS<var> <val>] [-np]\n"
    "                    [-md5] [-OD<cgi>] [-OS<cgi>] [-version] [-x]";
 
