@@ -683,7 +683,7 @@ int main( int argc, char **argv )
       char *cwd = getcwd( buf, FILENAME_MAX );
       if( !cwd )
       {
-        XRootDStatus st( stError, XProtocol::mapError( errno ), errno, XrdSysE2T( errno ) );
+        XRootDStatus st( stError, XProtocol::mapError( errno ), errno );
         std::cerr <<  st.GetErrorMessage() << std::endl;
         return st.GetShellCode();
       }
@@ -710,7 +710,7 @@ int main( int argc, char **argv )
     URL target( dest );
     FileSystem fs( target );
     StatInfo *statInfo = 0;
-    XRootDStatus st = fs.Stat( target.GetPath(), statInfo );
+    XRootDStatus st = fs.Stat( target.GetPathWithParams(), statInfo );
     if( st.IsOK() )
     {
       if( statInfo->TestFlags( StatInfo::IsDir ) )
@@ -723,6 +723,12 @@ int main( int argc, char **argv )
       if( config.dstFile->Path[n-1] == '/' )
         targetIsDir = true;
     }
+    else if( st.errNo == kXR_NotAuthorized )
+    {
+      log->Error( AppMsg, "%s (destination)", st.ToString().c_str() );
+      std::cerr << st.ToStr() << std::endl;
+      return st.GetShellCode();
+    }
 
     delete statInfo;
   }
@@ -806,7 +812,7 @@ int main( int argc, char **argv )
         char *cwd = getcwd( buf, FILENAME_MAX );
         if( !cwd )
         {
-          XRootDStatus st( stError, XProtocol::mapError( errno ), errno, XrdSysE2T( errno ) );
+          XRootDStatus st( stError, XProtocol::mapError( errno ), errno );
           std::cerr <<  st.GetErrorMessage() << std::endl;
           return st.GetShellCode();
         }
@@ -871,7 +877,7 @@ int main( int argc, char **argv )
     properties.Set( "targetIsDir",     targetIsDir            );
     properties.Set( "preserveXAttr",   preserveXAttr          );
     properties.Set( "xrate",           config.xRate           );
-    properties.Set( "xrateThreashold", config.xRateThreashold );
+    properties.Set( "xrateThreshold",  config.xRateThreshold  );
     properties.Set( "rmOnBadCksum",    rmOnBadCksum           );
     properties.Set( "continue",        continue_              );
     properties.Set( "zipAppend",       zipappend              );
