@@ -152,11 +152,10 @@ namespace
         int rc = aio_return( me->cb.get() );
         if( rc < 0 )
         {
+          int errcode = aio_error( me->cb.get() );
           Log *log = DefaultEnv::GetLog();
-          log->Error( FileMsg, GetErrMsg( me->opcode ), XrdSysE2T( errno ) );
-          XRootDStatus *error = new XRootDStatus( stError, errErrorResponse,
-                                                  XProtocol::mapError( errno ),
-                                                  XrdSysE2T( errno ) );
+          log->Error( FileMsg, GetErrMsg( me->opcode ), XrdSysE2T( errcode ) );
+          XRootDStatus *error = new XRootDStatus( stError, errLocalError, errcode ) ;
           QueueTask( error, 0, me->hosts, me->handler );
         }
         else
@@ -202,7 +201,7 @@ namespace
         // and return there is no need to execute this in the thread-pool
         SyncResponseHandler *syncHandler =
             dynamic_cast<SyncResponseHandler*>( handler );
-        if( syncHandler )
+        if( syncHandler || DefaultEnv::GetPostMaster() == nullptr )
         {
           syncHandler->HandleResponse( status, resp );
         }
@@ -231,7 +230,6 @@ namespace XrdCl
   LocalFileHandler::LocalFileHandler() :
       fd( -1 )
   {
-    jmngr = DefaultEnv::GetPostMaster()->GetJobManager();
   }
 
   //------------------------------------------------------------------------
@@ -250,7 +248,7 @@ namespace XrdCl
   {
     AnyObject *resp = 0;
     XRootDStatus st = OpenImpl( url, flags, mode, resp );
-    if( !st.IsOK() && st.code != errErrorResponse )
+    if( !st.IsOK() && st.code != errLocalError )
       return st;
 
     return QueueTask( new XRootDStatus( st ), resp, handler );
@@ -275,9 +273,7 @@ namespace XrdCl
     {
       Log *log = DefaultEnv::GetLog();
       log->Error( FileMsg, "Close: file fd: %i %s", fd, XrdSysE2T( errno ) );
-      XRootDStatus *error = new XRootDStatus( stError, errErrorResponse,
-                                              XProtocol::mapError( errno ),
-                                              XrdSysE2T( errno ) );
+      XRootDStatus *error = new XRootDStatus( stError, errLocalError, errno );
       return QueueTask( error, 0, handler );
     }
 
@@ -296,9 +292,7 @@ namespace XrdCl
     if( fstat( fd, &ssp ) == -1 )
     {
       log->Error( FileMsg, "Stat: failed fd: %i %s", fd, XrdSysE2T( errno ) );
-      XRootDStatus *error = new XRootDStatus( stError, errErrorResponse,
-                                              XProtocol::mapError( errno ),
-                                              XrdSysE2T( errno ) );
+      XRootDStatus *error = new XRootDStatus( stError, errLocalError, errno );
       return QueueTask( error, 0, handler );
     }
     std::ostringstream data;
@@ -311,7 +305,7 @@ namespace XrdCl
     {
       log->Error( FileMsg, "Stat: ParseServerResponse failed." );
       delete statInfo;
-      return QueueTask( new XRootDStatus( stError, errErrorResponse, kXR_FSError ),
+      return QueueTask( new XRootDStatus( stError, errLocalError, kXR_FSError ),
                         0, handler );
     }
 
@@ -332,9 +326,7 @@ namespace XrdCl
     if( ( read = pread( fd, buffer, size, offset ) ) == -1 )
     {
       log->Error( FileMsg, "Read: failed %s", XrdSysE2T( errno ) );
-      XRootDStatus *error = new XRootDStatus( stError, errErrorResponse,
-                                              XProtocol::mapError( errno ),
-                                              XrdSysE2T( errno ) );
+      XRootDStatus *error = new XRootDStatus( stError, errLocalError, errno );
       return QueueTask( error, 0, handler );
     }
     ChunkInfo *chunk = new ChunkInfo( offset, read, buffer );
@@ -351,8 +343,7 @@ namespace XrdCl
     {
       Log *log = DefaultEnv::GetLog();
       log->Error( FileMsg, "Read: failed %s", XrdSysE2T( errno ) );
-      return XRootDStatus( stError, errOSError, XProtocol::mapError( rc ),
-                           XrdSysE2T( errno ) );
+      return XRootDStatus( stError, errLocalError, errno );
     }
 
     return XRootDStatus();
@@ -380,9 +371,7 @@ namespace XrdCl
     if( ret == -1 )
     {
       log->Error( FileMsg, "ReadV: failed %s", XrdSysE2T( errno ) );
-      XRootDStatus *error = new XRootDStatus( stError, errErrorResponse,
-                                              XProtocol::mapError( errno ),
-                                              XrdSysE2T( errno ) );
+      XRootDStatus *error = new XRootDStatus( stError, errLocalError, errno );
       return QueueTask( error, 0, handler );
     }
     VectorReadInfo *info = new VectorReadInfo();
@@ -418,9 +407,7 @@ namespace XrdCl
       {
         Log *log = DefaultEnv::GetLog();
         log->Error( FileMsg, "Write: failed %s", XrdSysE2T( errno ) );
-        XRootDStatus *error = new XRootDStatus( stError, errErrorResponse,
-                                                XProtocol::mapError( errno ),
-                                                XrdSysE2T( errno ) );
+        XRootDStatus *error = new XRootDStatus( stError, errLocalError, errno );
         return QueueTask( error, 0, handler );
       }
       offset += ret;
@@ -438,8 +425,7 @@ namespace XrdCl
     {
       Log *log = DefaultEnv::GetLog();
       log->Error( FileMsg, "Write: failed %s", XrdSysE2T( errno ) );
-      return XRootDStatus( stError, errOSError, XProtocol::mapError( rc ),
-                           XrdSysE2T( errno ) );
+      return XRootDStatus( stError, errLocalError, errno );
     }
 
     return XRootDStatus();
@@ -471,8 +457,7 @@ namespace XrdCl
     {
       Log *log = DefaultEnv::GetLog();
       log->Error( FileMsg, "Sync: failed %s", XrdSysE2T( errno ) );
-      return XRootDStatus( stError, errOSError, XProtocol::mapError( rc ),
-                           XrdSysE2T( errno ) );
+      return XRootDStatus( stError, errLocalError, errno );
     }
 #endif
     return XRootDStatus();
@@ -489,9 +474,7 @@ namespace XrdCl
       Log *log = DefaultEnv::GetLog();
       log->Error( FileMsg, "Truncate: failed, file descriptor: %i, %s", fd,
                   XrdSysE2T( errno ) );
-      XRootDStatus *error = new XRootDStatus( stError, errErrorResponse,
-                                              XProtocol::mapError( errno ),
-                                              XrdSysE2T( errno ) );
+      XRootDStatus *error = new XRootDStatus( stError, errLocalError, errno );
       return QueueTask( error, 0, handler );
     }
 
@@ -520,9 +503,7 @@ namespace XrdCl
         Log *log = DefaultEnv::GetLog();
         log->Error( FileMsg, "VectorRead: failed, file descriptor: %i, %s",
                     fd, XrdSysE2T( errno ) );
-        XRootDStatus *error = new XRootDStatus( stError, errErrorResponse,
-                                                XProtocol::mapError( errno ),
-                                                XrdSysE2T( errno ) );
+        XRootDStatus *error = new XRootDStatus( stError, errLocalError, errno );
         return QueueTask( error, 0, handler );
       }
       totalSize += bytesRead;
@@ -554,9 +535,7 @@ namespace XrdCl
         Log *log = DefaultEnv::GetLog();
         log->Error( FileMsg, "VectorWrite: failed, file descriptor: %i, %s",
                     fd, XrdSysE2T( errno ) );
-        XRootDStatus *error = new XRootDStatus( stError, errErrorResponse,
-                                                XProtocol::mapError( errno ),
-                                                XrdSysE2T( errno ) );
+        XRootDStatus *error = new XRootDStatus( stError, errLocalError, errno );
         return QueueTask( error, 0, handler );
       }
     }
@@ -597,9 +576,7 @@ namespace XrdCl
       {
         Log *log = DefaultEnv::GetLog();
         log->Error( FileMsg, "WriteV: failed %s", XrdSysE2T( errno ) );
-        XRootDStatus *error = new XRootDStatus( stError, errErrorResponse,
-                                                XProtocol::mapError( errno ),
-                                                XrdSysE2T( errno ) );
+        XRootDStatus *error = new XRootDStatus( stError, errLocalError, errno );
         return QueueTask( error, 0, handler );
       }
 
@@ -658,7 +635,7 @@ namespace XrdCl
       std::string name  = std::get<xattr_name>( *itr );
       std::string value = std::get<xattr_value>( *itr );
       int err = xattr->Set( name.c_str(), value.c_str(), value.size(), 0, fd );
-      XRootDStatus status = err ? XRootDStatus( stError, XProtocol::mapError( -errno ) ) :
+      XRootDStatus status = err < 0 ? XRootDStatus( stError, errLocalError, -err ) :
                                   XRootDStatus();
 
       response.push_back( XAttrStatus( name, status ) );
@@ -696,7 +673,7 @@ namespace XrdCl
       if( ret >= 0 )
         value.append( buffer.get(), ret );
       else if( ret < 0 )
-        status = XRootDStatus( stError, XProtocol::mapError( -ret ) );
+        status = XRootDStatus( stError, errLocalError, -ret );
 
       response.push_back( XAttr( *itr, value, status ) );
     }
@@ -722,8 +699,8 @@ namespace XrdCl
     {
       std::string name = *itr;
       int err = xattr->Del( name.c_str(), 0, fd );
-      XRootDStatus status = err ? XRootDStatus( stError, XProtocol::mapError( -errno ) ) :
-                                  XRootDStatus();
+      XRootDStatus status = err < 0 ? XRootDStatus( stError, errLocalError, -err ) :
+                                      XRootDStatus();
 
       response.push_back( XAttrStatus( name, status ) );
     }
@@ -766,7 +743,7 @@ namespace XrdCl
       std::string value = ret >= 0 ? std::string( buffer.get(), ret ) :
                                      std::string();
       XRootDStatus status = ret >= 0 ? XRootDStatus() :
-                                       XRootDStatus( stError, XProtocol::mapError( -ret ) );
+                                       XRootDStatus( stError, errLocalError, -ret );
       response.push_back( XAttr( name, value, status ) );
     }
     xattr->Free( alist );
@@ -789,7 +766,7 @@ namespace XrdCl
     // and return there is no need to execute this in the thread-pool
     SyncResponseHandler *syncHandler =
         dynamic_cast<SyncResponseHandler*>( handler );
-    if( syncHandler )
+    if( syncHandler || DefaultEnv::GetPostMaster() == nullptr )
     {
       syncHandler->HandleResponse( st, resp );
       return XRootDStatus();
@@ -797,7 +774,7 @@ namespace XrdCl
 
     HostList *hosts = pHostList.empty() ? 0 : new HostList( pHostList );
     LocalFileTask *task = new LocalFileTask( st, resp, hosts, handler );
-    jmngr->QueueJob( task );
+    DefaultEnv::GetPostMaster()->GetJobManager()->QueueJob( task );
     return XRootDStatus();
   }
 
@@ -816,9 +793,7 @@ namespace XrdCl
       int rc = lstat( tmp.c_str(), &st );
       if( rc == 0 ) break;
       if( errno != ENOENT )
-        return XRootDStatus( stError, errErrorResponse,
-                             XProtocol::mapError( errno ),
-                             XrdSysE2T( errno ) );
+        return XRootDStatus( stError, errLocalError, errno );
       pos = path.rfind( '/', pos - 1 );
     }
 
@@ -829,9 +804,7 @@ namespace XrdCl
       if( mkdir( tmp.c_str(), 0755 ) )
       {
         if( errno != EEXIST )
-          return XRootDStatus( stError, errErrorResponse,
-                               XProtocol::mapError( errno ),
-                               XrdSysE2T( errno ) );
+          return XRootDStatus( stError, errLocalError, errno );
       }
       pos = path.find( '/', pos + 1 );
     }
@@ -892,9 +865,8 @@ namespace XrdCl
       log->Error( FileMsg, "Open: open failed: %s: %s", path.c_str(),
                   XrdSysE2T( errno ) );
 
-      return XRootDStatus( stError, errErrorResponse,
-                           XProtocol::mapError( errno ),
-                           XrdSysE2T( errno ) );
+      return XRootDStatus( stError, errLocalError,
+                           XProtocol::mapError( errno ) );
     }
     //---------------------------------------------------------------------
     // Stat File and cache statInfo in openInfo
@@ -903,9 +875,8 @@ namespace XrdCl
     if( fstat( fd, &ssp ) == -1 )
     {
       log->Error( FileMsg, "Open: stat failed." );
-      return XRootDStatus( stError, errErrorResponse,
-                           XProtocol::mapError( errno ),
-                           XrdSysE2T( errno ) );
+      return XRootDStatus( stError, errLocalError,
+                           XProtocol::mapError( errno ) );
     }
 
     std::ostringstream data;
@@ -917,7 +888,7 @@ namespace XrdCl
     {
       log->Error( FileMsg, "Open: ParseServerResponse failed." );
       delete statInfo;
-      return XRootDStatus( stError, errErrorResponse, kXR_FSError );
+      return XRootDStatus( stError, errLocalError, kXR_FSError );
     }
 
     // add the URL to hosts list
