@@ -57,7 +57,7 @@ namespace XrdCl
 
   struct PostMasterImpl
   {
-    PostMasterImpl() : pPoller( 0 ), pInitialized( false )
+    PostMasterImpl() : pPoller( 0 ), pInitialized( false ), pRunning( false )
     {
       Env *env = DefaultEnv::GetEnv();
       int workerThreads = DefaultWorkerThreads;
@@ -81,6 +81,7 @@ namespace XrdCl
     ChannelMap            pChannelMap;
     XrdSysMutex           pChannelMapMutex;
     bool                  pInitialized;
+    bool                  pRunning;
     JobManager           *pJobManager;
 
     XrdSysMutex           pMtx;
@@ -177,6 +178,7 @@ namespace XrdCl
       return false;
     }
 
+    pImpl->pRunning = true;
     return true;
   }
 
@@ -194,6 +196,7 @@ namespace XrdCl
       return false;
     if( !pImpl->pPoller->Stop() )
       return false;
+    pImpl->pRunning = false;
     return true;
   }
 
@@ -205,31 +208,14 @@ namespace XrdCl
     return true;
   }
 
-  //----------------------------------------------------------------------------
-  // Send a message synchronously
-  //----------------------------------------------------------------------------
-  XRootDStatus PostMaster::Send( const URL &url,
-                                 Message   *msg,
-                                 bool       stateful,
-                                 time_t     expires )
-  {
-    XrdSysRWLockHelper scopedLock( pImpl->pDisconnectLock );
-    Channel *channel = GetChannel( url );
-
-    if( !channel )
-      return XRootDStatus( stError, errNotSupported );
-
-    return channel->Send( msg, stateful, expires );
-  }
-
   //----------------------------------------------------------------------------
   // Send the message asynchronously
   //----------------------------------------------------------------------------
-  XRootDStatus PostMaster::Send( const URL            &url,
-                                 Message              *msg,
-                                 OutgoingMsgHandler   *handler,
-                                 bool                  stateful,
-                                 time_t                expires )
+  XRootDStatus PostMaster::Send( const URL    &url,
+                                 Message      *msg,
+                                 MsgHandler   *handler,
+                                 bool          stateful,
+                                 time_t        expires )
   {
     XrdSysRWLockHelper scopedLock( pImpl->pDisconnectLock );
     Channel *channel = GetChannel( url );
@@ -240,9 +226,9 @@ namespace XrdCl
     return channel->Send( msg, handler, stateful, expires );
   }
 
-  Status PostMaster::Redirect( const URL          &url,
-                               Message            *msg,
-                               IncomingMsgHandler *inHandler )
+  Status PostMaster::Redirect( const URL  &url,
+                               Message    *msg,
+                               MsgHandler *inHandler )
   {
     RedirectorRegistry &registry  = RedirectorRegistry::Instance();
     VirtualRedirector *redirector = registry.Get( url );
@@ -251,39 +237,6 @@ namespace XrdCl
     return redirector->HandleRequest( msg, inHandler );
   }
 
-  //----------------------------------------------------------------------------
-  // Synchronously receive a message
-  //----------------------------------------------------------------------------
-  Status PostMaster::Receive( const URL      &url,
-                              Message       *&msg,
-                              MessageFilter  *filter,
-                              time_t          expires )
-  {
-    XrdSysRWLockHelper scopedLock( pImpl->pDisconnectLock );
-    Channel *channel = GetChannel( url );
-
-    if( !channel )
-      return Status( stError, errNotSupported );
-
-    return channel->Receive( msg, filter, expires );
-  }
-
-  //----------------------------------------------------------------------------
-  // Listen to incoming messages
-  //----------------------------------------------------------------------------
-  Status PostMaster::Receive( const URL          &url,
-                              IncomingMsgHandler *handler,
-                              time_t              expires )
-  {
-    XrdSysRWLockHelper scopedLock( pImpl->pDisconnectLock );
-    Channel *channel = GetChannel( url );
-
-    if( !channel )
-      return Status( stError, errNotSupported );
-
-    return channel->Receive( handler, expires );
-  }
-
   //----------------------------------------------------------------------------
   // Query the transport handler
   //----------------------------------------------------------------------------
@@ -474,7 +427,7 @@ namespace XrdCl
     }
 
     Log *log = DefaultEnv::GetLog();
-    log->Info( PostMasterMsg, "Lable channel %s with alias %s.",
+    log->Info( PostMasterMsg, "Label channel %s with alias %s.",
                url.GetHostId().c_str(), alias.GetHostId().c_str() );
 
     Channel *active = new Channel( alias, pImpl->pPoller, trHandler,
@@ -486,6 +439,27 @@ namespace XrdCl
     //--------------------------------------------------------------------------
   }
 
+  //------------------------------------------------------------------------
+  // Decrement file object instance count bound to this channel
+  //------------------------------------------------------------------------
+  void PostMaster::DecFileInstCnt( const URL &url )
+  {
+    XrdSysRWLockHelper scopedLock( pImpl->pDisconnectLock );
+    Channel *channel = GetChannel( url );
+
+    if( !channel ) return;
+
+    return channel->DecFileInstCnt();
+  }
+
+  //------------------------------------------------------------------------
+  //true if underlying threads are running, false otherwise
+  //------------------------------------------------------------------------
+  bool PostMaster::IsRunning()
+  {
+    return pImpl->pRunning;
+  }
+
   //----------------------------------------------------------------------------
   // Get the channel
   //----------------------------------------------------------------------------
