@@ -261,6 +261,8 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   void Socket::Close()
   {
+    if( pTls ) pTls->Shutdown();
+
     if( pSocket != -1 )
     {
       close( pSocket );
@@ -482,7 +484,7 @@ namespace XrdCl
   //! @param kbuff : data to be written
   //! @return      : the amount of data actually written
   //------------------------------------------------------------------------
-  XRootDStatus Socket::Send( XrdSys::KernelBuffer &kbuff )
+  XRootDStatus Socket::Send( XrdSys::KernelBuffer &kbuff, int &bytesWritten )
   {
     if( pTls ) return XRootDStatus( stError, errNotSupported, 0,
                                      "Cannot send a kernel-buffer over TLS." );
@@ -491,6 +493,45 @@ namespace XrdCl
 
     if( status <= 0 )
       return ClassifyErrno( errno );
+    bytesWritten += status;
+    return XRootDStatus();
+  }
+
+  //------------------------------------------------------------------------
+  // Write message to the socket
+  //------------------------------------------------------------------------
+  XRootDStatus Socket::Send( Message &msg, const std::string &strmname )
+  {
+    //----------------------------------------------------------------------
+    // Try to write down the current message
+    //----------------------------------------------------------------------
+    size_t btsleft = msg.GetSize() - msg.GetCursor();
+    if( !btsleft ) return XRootDStatus();
+
+    while( btsleft )
+    {
+      int wrtcnt = 0;
+      XRootDStatus st = Send( msg.GetBufferAtCursor(), btsleft, wrtcnt );
+
+      if( !st.IsOK() )
+      {
+        msg.SetCursor( 0 );
+        return st;
+      }
+
+      if( st.code == suRetry ) return st;
+
+      msg.AdvanceCursor( wrtcnt );
+      btsleft -= wrtcnt;
+    }
+
+    //----------------------------------------------------------------------
+    // We have written the message successfully
+    //----------------------------------------------------------------------
+    Log *log = DefaultEnv::GetLog();
+    log->Dump( AsyncSockMsg, "[%s] Wrote a message: %s (0x%x), %d bytes",
+               strmname.c_str(), msg.GetDescription().c_str(),
+               &msg, msg.GetSize() );
     return XRootDStatus();
   }
 
@@ -714,6 +755,27 @@ namespace XrdCl
     return XRootDStatus();
   }
 
+  //----------------------------------------------------------------------------
+  // ReadV helper for raw socket
+  //----------------------------------------------------------------------------
+  XRootDStatus Socket::ReadV( iovec *iov, int iovcnt, int &bytesRead )
+  {
+    if( pTls ) return pTls->ReadV( iov, iovcnt, bytesRead );
+
+    int status = ::readv( pSocket, iov, iovcnt );
+
+    // if the server shut down the socket declare a socket error (it
+    // will trigger a re-connect)
+    if( status == 0 )
+      return XRootDStatus( stError, errSocketError, errno );
+
+    if( status < 0 )
+      return ClassifyErrno( errno );
+
+    bytesRead = status;
+    return XRootDStatus();
+  }
+
   //------------------------------------------------------------------------
   // Cork the underlying socket
   //------------------------------------------------------------------------
