@@ -123,6 +123,17 @@ namespace XrdCl
                                                       DefaultNetworkStack );
 
     pAddressType = Utils::String2AddressType( netStack );
+    if( pAddressType == Utils::AddressType::IPAuto )
+    {
+      XrdNetUtils::NetProt stacks = XrdNetUtils::NetConfig( XrdNetUtils::NetType::qryINIF );
+      if( !( stacks & XrdNetUtils::hasIP64 ) )
+      {
+        if( stacks & XrdNetUtils::hasIPv4 )
+          pAddressType = Utils::AddressType::IPv4;
+        else if( stacks & XrdNetUtils::hasIPv6 )
+          pAddressType = Utils::AddressType::IPv6;
+      }
+    }
 
     Log *log = DefaultEnv::GetLog();
     log->Debug( PostMasterMsg, "[%s] Stream parameters: Network Stack: %s, "
@@ -456,45 +467,42 @@ namespace XrdCl
     msg->SetSessionId( pSessionId );
     pBytesReceived += bytesReceived;
 
-    uint32_t streamAction = pTransport->MessageReceived( *msg, subStream,
-                                                         *pChannelData );
-    if( streamAction & TransportHandler::DigestMsg )
-      return;
-
-    if( streamAction & TransportHandler::RequestClose )
+    MsgHandler *handler = nullptr;
+    uint16_t action = 0;
     {
-      RequestClose( *msg );
-      return;
+      InMessageHelper &mh = pSubStreams[subStream]->inMsgHelper;
+      handler = mh.handler;
+      action = mh.action;
+      mh.Reset();
+    }
+
+    if( !IsPartial( *msg ) )
+    {
+      uint32_t streamAction = pTransport->MessageReceived( *msg, subStream,
+                                                           *pChannelData );
+      if( streamAction & TransportHandler::DigestMsg )
+        return;
+
+      if( streamAction & TransportHandler::RequestClose )
+      {
+        RequestClose( *msg );
+        return;
+      }
     }
 
     Log *log = DefaultEnv::GetLog();
-    InMessageHelper &mh = pSubStreams[subStream]->inMsgHelper;
 
     //--------------------------------------------------------------------------
-    // Check if we can obtain a handler now that we have the whole response
+    // No handler, we discard the message ...
     //--------------------------------------------------------------------------
-    if( !mh.handler )
+    if( !handler )
     {
-      //------------------------------------------------------------------------
-      // Try once more if we can obtain a handler for the message, for the async
-      // (kXR_attn) messages this can be only done once the whole message has
-      // been read out from the socket
-      //------------------------------------------------------------------------
-      mh.handler = pIncomingQueue->GetHandlerForMessage( msg,
-                                                         mh.expires,
-                                                         mh.action );
-      //------------------------------------------------------------------------
-      // No handler, we discard the message ...
-      //------------------------------------------------------------------------
-      if( !mh.handler )
-      {
-        ServerResponse *rsp = (ServerResponse*)msg->GetBuffer();
-        log->Warning( PostMasterMsg, "[%s] Discarding received message: 0x%x "
-                      "(status=%d, SID=[%d,%d]), no MsgHandler found.",
-                      pStreamName.c_str(), msg.get(), rsp->hdr.status,
-                      rsp->hdr.streamid[0], rsp->hdr.streamid[1] );
-        return;
-      }
+      ServerResponse *rsp = (ServerResponse*)msg->GetBuffer();
+      log->Warning( PostMasterMsg, "[%s] Discarding received message: 0x%x "
+                    "(status=%d, SID=[%d,%d]), no MsgHandler found.",
+                    pStreamName.c_str(), msg.get(), rsp->hdr.status,
+                    rsp->hdr.streamid[0], rsp->hdr.streamid[1] );
+      return;
     }
 
     //--------------------------------------------------------------------------
@@ -503,10 +511,7 @@ namespace XrdCl
     log->Dump( PostMasterMsg, "[%s] Handling received message: 0x%x.",
                pStreamName.c_str(), msg.get() );
 
-    if( !(mh.action & MsgHandler::RemoveHandler) )
-      pIncomingQueue->ReAddMessageHandler( mh.handler, mh.expires );
-
-    if( mh.action & (MsgHandler::NoProcess|MsgHandler::Ignore) )
+    if( action & (MsgHandler::NoProcess|MsgHandler::Ignore) )
     {
       log->Dump( PostMasterMsg, "[%s] Ignoring the processing handler for: 0x%x.",
                  pStreamName.c_str(), msg->GetDescription().c_str() );
@@ -514,16 +519,14 @@ namespace XrdCl
       // if we are handling partial response we have to take down the timeout fence
       if( IsPartial( *msg ) )
       {
-        XRootDMsgHandler *xrdHandler = dynamic_cast<XRootDMsgHandler*>( mh.handler );
+        XRootDMsgHandler *xrdHandler = dynamic_cast<XRootDMsgHandler*>( handler );
         if( xrdHandler ) xrdHandler->PartialReceived();
       }
 
-      mh.Reset();
       return;
     }
 
-    Job *job = new HandleIncMsgJob( mh.handler );
-    mh.Reset();
+    Job *job = new HandleIncMsgJob( handler );
     pJobManager->QueueJob( job );
   }
 
@@ -600,9 +603,11 @@ namespace XrdCl
   {
     XrdSysMutexHelper scopedLock( pMutex );
     pSubStreams[subStream]->status = Socket::Connected;
+
+    std::string ipstack( pSubStreams[0]->socket->GetIpStack() );
     Log *log = DefaultEnv::GetLog();
-    log->Debug( PostMasterMsg, "[%s] Stream %d connected.", pStreamName.c_str(),
-                subStream );
+    log->Debug( PostMasterMsg, "[%s] Stream %d connected (%s).", pStreamName.c_str(),
+                subStream, ipstack.c_str() );
 
     if( subStream == 0 )
     {
@@ -815,12 +820,14 @@ namespace XrdCl
     }
 
     //--------------------------------------------------------------------------
-    // Reinsert the receiving handler
+    // Reinsert the receiving handler and reset any partially read partial
     //--------------------------------------------------------------------------
     if( pSubStreams[subStream]->inMsgHelper.handler )
     {
       InMessageHelper &h = pSubStreams[subStream]->inMsgHelper;
       pIncomingQueue->ReAddMessageHandler( h.handler, h.expires );
+      XRootDMsgHandler *xrdHandler = dynamic_cast<XRootDMsgHandler*>( h.handler );
+      if( xrdHandler ) xrdHandler->PartialReceived();
       h.Reset();
     }
 
@@ -898,9 +905,9 @@ namespace XrdCl
   void Stream::ForceError( XRootDStatus status )
   {
     XrdSysMutexHelper scopedLock( pMutex );
+    Log    *log = DefaultEnv::GetLog();
     for( size_t substream = 0; substream < pSubStreams.size(); ++substream )
     {
-      Log    *log = DefaultEnv::GetLog();
       if( pSubStreams[substream]->status != Socket::Connected ) continue;
       pSubStreams[substream]->socket->Close();
       pSubStreams[substream]->status = Socket::Disconnected;
@@ -919,35 +926,37 @@ namespace XrdCl
       }
 
       //--------------------------------------------------------------------
-      // Reinsert the receiving handler
+      // Reinsert the receiving handler and reset any partially read partial
       //--------------------------------------------------------------------
       if( pSubStreams[substream]->inMsgHelper.handler )
       {
         InMessageHelper &h = pSubStreams[substream]->inMsgHelper;
         pIncomingQueue->ReAddMessageHandler( h.handler, h.expires );
+        XRootDMsgHandler *xrdHandler = dynamic_cast<XRootDMsgHandler*>( h.handler );
+        if( xrdHandler ) xrdHandler->PartialReceived();
         h.Reset();
       }
-
-      pConnectionCount = 0;
-
-      //------------------------------------------------------------------------
-      // We're done here, unlock the stream mutex to avoid deadlocks and
-      // report the disconnection event to the handlers
-      //------------------------------------------------------------------------
-      log->Debug( PostMasterMsg, "[%s] Reporting disconnection to queued "
-                  "message handlers.", pStreamName.c_str() );
-
-      SubStreamList::iterator it;
-      OutQueue q;
-      for( it = pSubStreams.begin(); it != pSubStreams.end(); ++it )
-        q.GrabItems( *(*it)->outQueue );
-      scopedLock.UnLock();
-
-      q.Report( status );
-
-      pIncomingQueue->ReportStreamEvent( MsgHandler::Broken, status );
-      pChannelEvHandlers.ReportEvent( ChannelEventHandler::StreamBroken, status );
     }
+
+    pConnectionCount = 0;
+
+    //------------------------------------------------------------------------
+    // We're done here, unlock the stream mutex to avoid deadlocks and
+    // report the disconnection event to the handlers
+    //------------------------------------------------------------------------
+    log->Debug( PostMasterMsg, "[%s] Reporting disconnection to queued "
+                "message handlers.", pStreamName.c_str() );
+
+    SubStreamList::iterator it;
+    OutQueue q;
+    for( it = pSubStreams.begin(); it != pSubStreams.end(); ++it )
+      q.GrabItems( *(*it)->outQueue );
+    scopedLock.UnLock();
+
+    q.Report( status );
+
+    pIncomingQueue->ReportStreamEvent( MsgHandler::Broken, status );
+    pChannelEvHandlers.ReportEvent( ChannelEventHandler::StreamBroken, status );
   }
 
   //----------------------------------------------------------------------------
@@ -1007,10 +1016,8 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   // Call back when a message has been reconstructed
   //----------------------------------------------------------------------------
-  void Stream::OnReadTimeout( uint16_t substream, bool &isBroken )
+  void Stream::OnReadTimeout( uint16_t substream )
   {
-    isBroken = false;
-
     //--------------------------------------------------------------------------
     // We only take the main stream into account
     //--------------------------------------------------------------------------
@@ -1066,7 +1073,6 @@ namespace XrdCl
                                             *pChannelData );
     if( !st.IsOK() )
     {
-      isBroken =  true;
       scopedLock.UnLock();
       OnError( substream, st );
     }
@@ -1124,7 +1130,8 @@ namespace XrdCl
                                      MsgHandler *&incHandler )
   {
     InMessageHelper &mh = pSubStreams[stream]->inMsgHelper;
-    if( !mh.handler ) return false;
+    if( !mh.handler )
+      return MsgHandler::RemoveHandler;
 
     uint16_t action = mh.handler->InspectStatusRsp();
     mh.action |= action;
@@ -1191,4 +1198,35 @@ namespace XrdCl
 
     return false;
   }
+
+  //------------------------------------------------------------------------
+  // Query the stream
+  //------------------------------------------------------------------------
+  Status Stream::Query( uint16_t   query, AnyObject &result )
+  {
+    switch( query )
+    {
+      case StreamQuery::IpAddr:
+      {
+        result.Set( new std::string( pSubStreams[0]->socket->GetIpAddr() ), false );
+        return Status();
+      }
+
+      case StreamQuery::IpStack:
+      {
+        result.Set( new std::string( pSubStreams[0]->socket->GetIpStack() ), false );
+        return Status();
+      }
+
+      case StreamQuery::HostName:
+      {
+        result.Set( new std::string( pSubStreams[0]->socket->GetHostName() ), false );
+        return Status();
+      }
+
+      default:
+        return Status( stError, errQueryNotSupported );
+    }
+  }
+
 }
