@@ -22,6 +22,7 @@
 #include "XrdCl/XrdClConstants.hh"
 #include "XrdCl/XrdClCheckSumManager.hh"
 #include "XrdCl/XrdClRedirectorRegistry.hh"
+#include "XrdCl/XrdClMessage.hh"
 #include "XrdNet/XrdNetAddr.hh"
 
 #include <algorithm>
@@ -42,7 +43,7 @@
 #include <dirent.h>
 
 #if __cplusplus < 201103L
-#include <time.h>
+#include <ctime>
 #endif
 
 namespace
@@ -140,9 +141,8 @@ namespace XrdCl
                                   Utils::AddressType      type )
   {
     Log *log = DefaultEnv::GetLog();
-    XrdNetAddr *addrs;
-    int         nAddrs = 0;
     const char *err    = 0;
+    int ordn;
 
     //--------------------------------------------------------------------------
     // Resolve all the addresses
@@ -156,22 +156,6 @@ namespace XrdCl
     else if( type == IPAll ) opts = XrdNetUtils::allIPMap;
     else opts = XrdNetUtils::prefAuto;
 
-    err = XrdNetUtils::GetAddrs( o.str().c_str(), &addrs, nAddrs, opts );
-
-    if( err )
-    {
-      log->Error( UtilityMsg, "Unable to resolve %s: %s", o.str().c_str(),
-                  err );
-      return Status( stError, errInvalidAddr );
-    }
-
-    if( nAddrs == 0 )
-    {
-      log->Error( UtilityMsg, "No addresses for %s were found",
-                  o.str().c_str() );
-      return Status( stError, errInvalidAddr );
-    }
-
     //--------------------------------------------------------------------------
     // Check what are the preferences IPv6 or IPv4
     //--------------------------------------------------------------------------
@@ -184,52 +168,62 @@ namespace XrdCl
     // The preferred IP family goes to the back as it is easier to remove
     // items from the back of the vector
     //--------------------------------------------------------------------------
-    std::vector<XrdNetAddr> result( nAddrs );
-    auto itr  = result.begin();
-    auto ritr = result.end() - 1;
+    opts |= (preferIPv4 ? XrdNetUtils::order64 : XrdNetUtils::order46);
 
-    for( int i = 0; i < nAddrs; ++i )
+    //--------------------------------------------------------------------------
+    // Now get all of the properly partitioned addresses; ordn will hold the
+    // number of non-preferred addresses at the front of the table.
+    //--------------------------------------------------------------------------
+    err = XrdNetUtils::GetAddrs( o.str(), addresses, &ordn, opts );
+
+    if( err )
     {
-      bool isIPv4 = addrs[i].isIPType( XrdNetAddrInfo::IPv4 ) ||
-          ( addrs[i].isIPType( XrdNetAddrInfo::IPv6 ) && addrs[i].isMapped() );
+      log->Error( UtilityMsg, "Unable to resolve %s: %s", o.str().c_str(),
+                  err );
+      return Status( stError, errInvalidAddr );
+    }
 
-      auto store = preferIPv4 ?
-                   ( isIPv4 ? ritr-- : itr++ ) :
-                   ( isIPv4 ? itr++ : ritr-- );
-
-      *store = addrs[i];
+    if( addresses.size() == 0 )
+    {
+      log->Error( UtilityMsg, "No addresses for %s were found",
+                  o.str().c_str() );
+      return Status( stError, errInvalidAddr );
     }
 
     //--------------------------------------------------------------------------
     // Shuffle each partition
     //--------------------------------------------------------------------------
-#if __cplusplus < 201103L
 
-    // initialize the random generator only once
-    static struct only_once_t
+    int ipNoShuffle = DefaultIPNoShuffle;
+    Env *env = DefaultEnv::GetEnv();
+    env->GetInt( "IPNoShuffle", ipNoShuffle );
+
+    if( !ipNoShuffle )
     {
-      only_once_t()
+#if __cplusplus < 201103L
+      // initialize the random generator only once
+      static struct only_once_t
       {
-        std::srand ( unsigned ( std::time(0) ) );
-      }
-    } only_once;
+        only_once_t()
+        {
+          std::srand ( unsigned ( std::time(0) ) );
+        }
+      } only_once;
 
-    std::random_shuffle( result.begin(), itr );
-    std::random_shuffle( itr, result.end() );
+      std::random_shuffle( addresses.begin(), addresses.begin() + ordn );
+      std::random_shuffle( addresses.begin() + ordn, addresses.end() );
 #else
-    static std::default_random_engine rand_engine(
-        std::chrono::system_clock::now().time_since_epoch().count() );
+      static std::default_random_engine rand_engine(
+          std::chrono::system_clock::now().time_since_epoch().count() );
 
-    std::shuffle( result.begin(), itr, rand_engine );
-    std::shuffle( itr, result.end(), rand_engine );
+      std::shuffle( addresses.begin(), addresses.begin() + ordn, rand_engine );
+      std::shuffle( addresses.begin() + ordn, addresses.end(), rand_engine );
 #endif
+    }
 
     //--------------------------------------------------------------------------
-    // Return result through output parameter
+    // Return status as the result is already in the output parameter
     //--------------------------------------------------------------------------
-    addresses.swap( result );
-    delete [] addrs;
-
     return Status();
   }
 
@@ -283,15 +277,14 @@ namespace XrdCl
   //----------------------------------------------------------------------------
   XRootDStatus Utils::GetRemoteCheckSum( std::string       &checkSum,
                                          const std::string &checkSumType,
-                                         const std::string &server,
-                                         const std::string &path )
+                                         const URL         &url )
   {
-    FileSystem   *fs = new FileSystem( URL( server ) );
+    FileSystem   *fs = new FileSystem( url );
     // add the 'cks.type' cgi tag in order to
     // select the proper checksum type in case
     // the server supports more than one checksum
-    size_t pos = path.find( '?' );
-    std::string cksPath = path + ( pos == std::string::npos ? '?' : '&' ) + "cks.type=" + checkSumType;
+    size_t pos = url.GetPath().find( '?' );
+    std::string cksPath = url.GetPath() + ( pos == std::string::npos ? '?' : '&' ) + "cks.type=" + checkSumType;
     Buffer        arg; arg.FromString( cksPath );
     Buffer       *cksResponse = 0;
     XRootDStatus  st;
@@ -325,7 +318,7 @@ namespace XrdCl
     checkSum += NormalizeChecksum( elems[0], elems[1] );
 
     log->Dump( UtilityMsg, "Checksum for %s checksum: %s",
-               path.c_str(), checkSum.c_str() );
+               url.GetPath().c_str(), checkSum.c_str() );
 
     return XRootDStatus();
   }
@@ -566,6 +559,46 @@ namespace XrdCl
     return Status();
   }
 
+  //------------------------------------------------------------------------
+  //! Process a config directory and return key-value pairs
+  //------------------------------------------------------------------------
+  Status Utils::ProcessConfigDir( std::map<std::string, std::string> &config,
+                                  const std::string                  &dir )
+  {
+    Log *log = DefaultEnv::GetLog();
+    log->Debug( UtilityMsg, "Processing configuration files in %s...",
+                dir.c_str());
+
+    std::vector<std::string> entries;
+    Status st = Utils::GetDirectoryEntries( entries, dir );
+    if( !st.IsOK() )
+    {
+      log->Debug( UtilityMsg, "Unable to process directory %s: %s",
+                  dir.c_str(), st.ToString().c_str() );
+      return st;
+    }
+
+    static const std::string suffix   = ".conf";
+    for( auto &entry : entries )
+    {
+      std::string confFile = dir + "/" + entry;
+
+      if( confFile.length() <= suffix.length() )
+        continue;
+      if( !std::equal( suffix.rbegin(), suffix.rend(), confFile.rbegin() ) )
+        continue;
+
+      st = ProcessConfig( config, confFile );
+      if( !st.IsOK() )
+      {
+        log->Debug( UtilityMsg, "Unable to process configuration file %s: %s",
+                    confFile.c_str(), st.ToString().c_str() );
+      }
+    }
+
+    return Status();
+  }
+
   //----------------------------------------------------------------------------
   // Trim a string
   //----------------------------------------------------------------------------
@@ -661,6 +694,74 @@ namespace XrdCl
   }
 
 
+  //------------------------------------------------------------------------
+  //! Check if this client can support given EC redirect
+  //------------------------------------------------------------------------
+  bool Utils::CheckEC( const Message *req, const URL &url )
+  {
+#ifdef WITH_XRDEC
+    // make sure that if we will be writing it is a new file
+    ClientRequest *request = (ClientRequest*)req->GetBuffer();
+    uint16_t options = ntohs( request->open.options );
+    bool open_wrt = ( options & kXR_open_updt ) || ( options & kXR_open_wrto );
+    bool open_new = ( options & kXR_new );
+    if( open_wrt && !open_new ) return false;
+
+    const URL::ParamsMap &params = url.GetParams();
+    // make sure all the xrdec. tokens are present and the values are sane
+    URL::ParamsMap::const_iterator itr = params.find( "xrdec.nbdta" );
+    if( itr == params.end() ) return false;
+    size_t nbdta = std::stoul( itr->second );
+
+    itr = params.find( "xrdec.nbprt" );
+    if( itr == params.end() ) return false;
+    size_t nbprt = std::stoul( itr->second );
+
+    itr = params.find( "xrdec.blksz" );
+    if( itr == params.end() ) return false;
+
+    itr = params.find( "xrdec.plgr" );
+    if( itr == params.end() ) return false;
+    std::vector<std::string> plgr;
+    splitString( plgr, itr->second, "," );
+    if( plgr.size() < nbdta + nbprt ) return false;
+
+    itr = params.find( "xrdec.objid" );
+    if( itr == params.end() ) return false;
+
+    itr = params.find( "xrdec.format" );
+    if( itr == params.end() ) return false;
+    size_t format = std::stoul( itr->second );
+    if( format != 1 ) return false; // TODO use constant
+
+    itr = params.find( "xrdec.dtacgi" );
+    if( itr != params.end() )
+    {
+      std::vector<std::string> dtacgi;
+      splitString( dtacgi, itr->second, "," );
+      if( plgr.size() != dtacgi.size() ) return false;
+    }
+
+    itr = params.find( "xrdec.mdtacgi" );
+    if( itr != params.end() )
+    {
+      std::vector<std::string> mdtacgi;
+      splitString( mdtacgi, itr->second, "," );
+      if( plgr.size() != mdtacgi.size() ) return false;
+    }
+
+    itr = params.find( "xrdec.cosc" );
+    if( itr == params.end() ) return false;
+    std::string cosc = itr->second;
+    if( cosc != "true" && cosc != "false" ) return false;
+
+    return true;
+#else
+    return false;
+#endif
+  }
+
+
   //----------------------------------------------------------------------------
   //! Automatically infer the right checksum type
   //----------------------------------------------------------------------------
@@ -669,7 +770,7 @@ namespace XrdCl
                                         bool              zip)
   {
     //--------------------------------------------------------------------------
-    // If both files are local we wont be checksumming at all
+    // If both files are local we won't be checksumming at all
     //--------------------------------------------------------------------------
     if( source.IsLocalFile() && !source.IsMetalink() && destination.IsLocalFile() ) return std::string();
 
