@@ -28,9 +28,9 @@
 /******************************************************************************/
 
 #define _FILE_OFFSET_BITS 64
-#include <errno.h>
-#include <stdio.h>
-#include <string.h>
+#include <cerrno>
+#include <cstdio>
+#include <cstring>
 #include <sys/types.h>
 
 #if !defined(__solaris__) && !defined(__FreeBSD__)
@@ -44,7 +44,7 @@
 #include <iostream>
 #include <libgen.h>
 #include <unistd.h>
-#include <stdlib.h>
+#include <cstdlib>
 #include <syslog.h>
 #include "XrdFfs/XrdFfsPosix.hh"
 #include "XrdPosix/XrdPosixXrootd.hh"
@@ -52,6 +52,11 @@
 #include "XrdFfs/XrdFfsDent.hh"
 #include "XrdFfs/XrdFfsQueue.hh"
 
+#include "XrdCl/XrdClFileSystem.hh"
+#include "XrdCl/XrdClFile.hh"
+#include "XrdCl/XrdClURL.hh"
+#include "XrdCl/XrdClXRootDResponses.hh"
+
 #ifdef __cplusplus
   extern "C" {
 #endif
@@ -91,7 +96,22 @@ int XrdFfsPosix_closedir(DIR *dirp)
 
 int XrdFfsPosix_mkdir(const char *path, mode_t mode)
 {
-    return XrdPosixXrootd::Mkdir(path, mode);
+    XrdCl::URL url(path);
+    std::string dir = url.GetPath();
+    XrdCl::LocationInfo *info = nullptr;
+    XrdCl::FileSystem fs(path);
+
+    XrdCl::XRootDStatus st = fs.DeepLocate("*", XrdCl::OpenFlags::None, info );
+    std::unique_ptr<XrdCl::LocationInfo> ptr( info );  
+
+    if( !st.IsOK() )
+    {
+        errno = ENOENT;
+        return -1;
+    }
+    std::string nodeUrl = "root://" + info->At(0).GetAddress() + "/" + dir;
+
+    return XrdPosixXrootd::Mkdir(nodeUrl.c_str(), mode);
 }
 
 int XrdFfsPosix_rmdir(const char *path)
@@ -800,8 +820,11 @@ int XrdFfsPosix_statall(const char *rdrurl, const char *path, struct stat *stbuf
     res = -1;
     errno = ENOENT;
     for (i = 0; i < nurls; i++)
+    {
+        time_t max_mtime = 0;
         if (res_i[i] == 0) 
         {
+            if (stbuf_i[i].st_mtime <= max_mtime) continue; 
             res = 0;
             errno = 0;
             memcpy((void*)stbuf, (void*)(&stbuf_i[i]), sizeof(struct stat));
@@ -813,6 +836,7 @@ int XrdFfsPosix_statall(const char *rdrurl, const char *path, struct stat *stbuf
             errno = ETIMEDOUT;
             syslog(LOG_WARNING, "WARNING: stat(%s) failed (connection timeout)", newurls[i]);
         }
+    }
 
     for (i = 0; i < nurls; i++)
         free(newurls[i]);
