@@ -30,12 +30,12 @@
 
 #ifndef WIN32
 #include <unistd.h>
-#include <ctype.h>
-#include <errno.h>
+#include <cctype>
+#include <cerrno>
 #include <fcntl.h>
 #include <poll.h>
-#include <stdio.h>
-#include <stdlib.h>
+#include <cstdio>
+#include <cstdlib>
 #include <strings.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
@@ -44,10 +44,10 @@
 #include <sys/stat.h>
 #include <sys/un.h>
 #else
-#include <errno.h>
+#include <cerrno>
 #include <fcntl.h>
-#include <stdio.h>
-#include <stdlib.h>
+#include <cstdio>
+#include <cstdlib>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <Winsock2.h>
@@ -347,7 +347,9 @@ int XrdNetSocket::setOpts(int xfd, int opts, XrdSysError *eDest)
 {
    int rc = 0;
    const int one = 1;
+#if defined(__linux__) || defined(__GNU__)
    const int szint = sizeof(int);
+#endif
    const SOCKLEN_t szone = (SOCKLEN_t)sizeof(one);
    static int tcpprotid = XrdNetUtils::ProtoID("tcp");
    static struct linger liopts = {1, XRDNETSOCKET_LINGER};
@@ -371,7 +373,7 @@ int XrdNetSocket::setOpts(int xfd, int opts, XrdSysError *eDest)
           {rc = 1;
            if (eDest) eDest->Emsg("setOpts", errno, "set socket KEEPALIVE");
           }
-#ifdef __linux__
+#if defined(__linux__) || defined(__GNU__)
            else if (opts & XRDNET_SERVER) // Following are inherited in Linux
       {if (XrdNetSocketCFG::ka_Idle
        &&  setsockopt(xfd,SOL_TCP,TCP_KEEPIDLE,&XrdNetSocketCFG::ka_Idle,szint))
@@ -424,6 +426,23 @@ int XrdNetSocket::setWindow(int xfd, int Windowsz, XrdSysError *eDest)
    return rc;
 }
 
+/******************************************************************************/
+/*                              S o c k N a m e                               */
+/******************************************************************************/
+
+int XrdNetSocket::SockName(char *buff, int blen)
+{
+
+// Make sure we have something here
+//
+   if (SockFD < 0) {*buff = 0; return ENOTSOCK;}
+
+// Format the name
+//
+   if (!SockInfo.Format(buff, blen)) return EINVAL;
+   return 0;
+}
+  
 /******************************************************************************/
 /*                            s o c k e t P a t h                             */
 /******************************************************************************/
