@@ -29,10 +29,10 @@
 /******************************************************************************/
 
 #include <unistd.h>
-#include <ctype.h>
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
+#include <cctype>
+#include <cstdlib>
+#include <cstring>
+#include <cstdio>
 #include <sys/param.h>
 
 #include "XrdVersion.hh"
