@@ -29,7 +29,7 @@
 /******************************************************************************/
   
 #include <signal.h>
-#include <stdio.h>
+#include <cstdio>
 #include <unistd.h>
 #include <fcntl.h>
 #ifdef _POSIX_ASYNCHRONOUS_IO
@@ -62,11 +62,17 @@
 #undef _POSIX_ASYNCHRONOUS_IO
 #endif
 
+#ifdef __GNU__
+// Compiler warning:
+// warning: sigwaitinfo is not implemented and will always fail
+#undef HAVE_SIGWTI
+#endif
+
 /******************************************************************************/
 /*                               G l o b a l s                                */
 /******************************************************************************/
   
-extern XrdOucTrace OssTrace;
+extern XrdSysTrace OssTrace;
 //define tident aiop->TIdent
 
 extern XrdSysError OssEroute;
@@ -158,9 +164,9 @@ int XrdOssFile::Read(XrdSfsAio *aiop)
       {aiop->sfsAio.aio_fildes = fd;
        aiop->sfsAio.aio_sigevent.sigev_signo  = OSS_AIO_READ_DONE;
        aiop->TIdent = tident;
-       TRACE(Debug,  "Read " <<aiop->sfsAio.aio_nbytes <<'@'
-                             <<aiop->sfsAio.aio_offset <<" started; aiocb="
-                             <<std::hex <<aiop <<std::dec);
+       TRACE(Debug,  "fd=" <<fd <<" read " <<aiop->sfsAio.aio_nbytes <<'@'
+                           <<aiop->sfsAio.aio_offset <<" started; aiocb="
+                           <<Xrd::hex1 <<aiop);
 
        // Start the operation
        //
@@ -215,9 +221,9 @@ int XrdOssFile::Write(XrdSfsAio *aiop)
       {aiop->sfsAio.aio_fildes = fd;
        aiop->sfsAio.aio_sigevent.sigev_signo  = OSS_AIO_WRITE_DONE;
        aiop->TIdent = tident;
-       TRACE(Debug, "Write " <<aiop->sfsAio.aio_nbytes <<'@'
-                             <<aiop->sfsAio.aio_offset <<" started; aiocb="
-                             <<std::hex <<aiop <<std::dec);
+       TRACE(Debug, "fd=" <<fd <<" write " <<aiop->sfsAio.aio_nbytes <<'@'
+                          <<aiop->sfsAio.aio_offset <<" started; aiocb="
+                          <<Xrd::hex1 <<aiop);
 
        // Start the operation
        //
@@ -409,7 +415,7 @@ void *XrdOssAioWait(void *mySigarg)
        retval = (ssize_t)aio_return(&aiop->sfsAio);
 
        DEBUG(sigType <<" completed for " <<aiop->TIdent <<"; rc=" <<rc 
-             <<" result=" <<retval <<" aiocb=" <<std::hex <<aiop <<std::dec);
+             <<" result=" <<retval <<" aiocb=" <<Xrd::hex1 <<aiop);
 
        if (retval < 0) aiop->Result = -rc;
           else         aiop->Result = retval;
