@@ -36,7 +36,7 @@ Configuration::Configuration() :
    m_accHistorySize(20),
    m_dirStatsMaxDepth(-1),
    m_dirStatsStoreDepth(0),
-   m_bufferSize(256*1024),
+   m_bufferSize(128*1024),
    m_RamAbsAvailable(0),
    m_RamKeepStdBlocks(0),
    m_wqueue_blocks(16),
@@ -241,7 +241,8 @@ bool Cache::xtrace(XrdOucStream &Config)
       {"warning", 2},
       {"info",    3},
       {"debug",   4},
-      {"dump",    5}
+      {"dump",    5},
+      {"dumpxl",  6}
    };
    int numopts = sizeof(tropts)/sizeof(struct traceopts);
 
@@ -303,7 +304,7 @@ bool Cache::Config(const char *config_filename, const char *parameters)
    // Adjust default parameters for client/serverless caching
    if (m_isClient)
    {
-      m_configuration.m_bufferSize     = 256 * 1024;
+      m_configuration.m_bufferSize     = 128 * 1024; // same as normal.
       m_configuration.m_wqueue_blocks  = 8;
       m_configuration.m_wqueue_threads = 1;
    }
@@ -532,6 +533,8 @@ bool Cache::Config(const char *config_filename, const char *parameters)
       }
 
       m_log.Say(buff);
+
+      m_env->Put("XRDPFC.SEGSIZE", std::to_string(m_configuration.m_bufferSize).c_str());
    }
 
    // Derived settings
