@@ -94,6 +94,7 @@ bool             oidsOK    = false;
 bool             p2lSRC    = false;
 bool             p2lSGI    = false;
 bool             autoPGRD  = false;
+bool             usingEC   = false;
 };
 
 int            XrdPosixXrootd::baseFD    = 0;
@@ -180,6 +181,12 @@ XrdPosixXrootd::XrdPosixXrootd(int fdnum, int dirnum, int thrnum)
    static XrdSysMutex myMutex;
    char *cfn;
 
+// Test if XRDCL_EC is set. That env var. is set at XrdCl::PlugInManager::LoadFactory
+// in XrdClPlugInManager.cc, which is called (by XrdOssGetSS while loading 
+// libXrdPss.so) before this function. 
+// Note: some standalone programs will call this constructor directly. 
+   XrdPosixGlobals::usingEC = getenv("XRDCL_EC")? true : false;
+
 // Only static fields are initialized here. We need to do this only once!
 //
    myMutex.Lock();
@@ -1060,8 +1067,11 @@ int XrdPosixXrootd::Rename(const char *oldpath, const char *newpath)
 
 // Issue the rename
 //
-  return XrdPosixMap::Result(admin.Xrd.Mv(admin.Url.GetPathWithParams(),
-                             newUrl.GetPathWithParams()));
+   if (XrdPosixGlobals::usingEC)
+       return EcRename(oldpath, newpath, &admin);
+
+   return XrdPosixMap::Result(admin.Xrd.Mv(admin.Url.GetPathWithParams(),
+                              newUrl.GetPathWithParams()));
 }
 
 /******************************************************************************/
@@ -1147,7 +1157,7 @@ int XrdPosixXrootd::Stat(const char *path, struct stat *buf)
 
 // Check if we can get the stat informatation from the cache
 //
-  if (XrdPosixGlobals::theCache)
+  if (! XrdPosixGlobals::usingEC && XrdPosixGlobals::theCache)
      {LfnPath statX("stat", path, false);
       if (!statX.path) return -1;
       int rc = XrdPosixGlobals::theCache->Stat(statX.path, *buf);
@@ -1157,9 +1167,12 @@ int XrdPosixXrootd::Stat(const char *path, struct stat *buf)
 
 // Issue the stat and verify that all went well
 //
+   if (XrdPosixGlobals::usingEC)
+       return EcStat(path, buf, &admin);
+
    if (!admin.Stat(*buf)) return -1;
    return 0;
-}
+}        
 
 /******************************************************************************/
 /*                                S t a t f s                                 */
@@ -1334,6 +1347,9 @@ int XrdPosixXrootd::Unlink(const char *path)
 
 // Issue the UnLink
 //
+   if (XrdPosixGlobals::usingEC)
+      return EcUnlink(path, &admin);
+
    return XrdPosixMap::Result(admin.Xrd.Rm(admin.Url.GetPathWithParams()));
 }
 
@@ -1464,3 +1480,215 @@ int XrdPosixXrootd::Fault(XrdPosixFile *fp, int ecode)
    errno = ecode;
    return -1;
 }
+
+/******************************************************************************/
+/*                               E c R e n a m e                              */
+/******************************************************************************/
+int XrdPosixXrootd::EcRename(const char *oldpath, const char *newpath, XrdPosixAdmin *admin)
+{
+    XrdCl::URL url(oldpath);
+    XrdCl::URL newUrl(newpath);
+
+    std::string file = url.GetPath();
+    XrdCl::LocationInfo *info = nullptr;
+    XrdCl::FileSystem fs(oldpath);
+
+    XrdCl::Buffer queryArgs(5), *queryResp = nullptr;
+    queryArgs.FromString("role");
+    XrdCl::XRootDStatus st = fs.Query(XrdCl::QueryCode::Config, queryArgs, queryResp);
+    // xrootdfs call this function with individual servers. but we can only do 
+    // fs.DeepLocate("*"...) agaist a redirector
+    // xrootdfs already did a stat and know that this is a file, not a dir
+    if (!st.IsOK() || queryResp->ToString() == "server"
+                   || queryResp->ToString() == "server\n")
+    {
+        if (queryResp) delete queryResp;
+        return XrdPosixMap::Result(admin->Xrd.Mv(admin->Url.GetPathWithParams(),
+                                   newUrl.GetPathWithParams()));
+    }
+    else
+        if (queryResp) delete queryResp;
+
+    st = fs.DeepLocate("*", XrdCl::OpenFlags::None, info );
+    std::unique_ptr<XrdCl::LocationInfo> ptr( info );
+    if( !st.IsOK() ) 
+    {
+        errno = ENOENT;
+        return -1;
+    }
+
+    // check if this is a file or a dir, do not support dir renaming in EC
+    struct stat buf;
+    if (XrdPosixXrootd::Stat(oldpath, &buf) != 0 || ! S_ISREG(buf.st_mode))
+    {
+        errno = ENOTSUP;
+        return -1;
+    }
+    if (XrdPosixXrootd::Stat(newpath, &buf) == 0) 
+    {
+        errno = ENOTSUP;
+        return -1;
+    }
+
+
+    int rc = -ENOENT;
+    for( size_t i = 0; i < info->GetSize(); ++i )
+    {
+        std::string url_i = "root://" + info->At(i).GetAddress() + "/" + file;
+        XrdPosixAdmin *admin_i = new XrdPosixAdmin(url_i.c_str());
+        int x = XrdPosixMap::Result(admin_i->Xrd.Mv(admin_i->Url.GetPathWithParams(),
+                                                    newUrl.GetPathWithParams()));
+        if (x != -ENOENT && rc != 0)
+            rc = x;
+        if (admin_i) delete admin_i;
+    }
+    return rc;
+}
+
+/******************************************************************************/
+/*                                  E c S t a t                               */
+/******************************************************************************/
+int XrdPosixXrootd::EcStat(const char *path, struct stat *buf, XrdPosixAdmin *admin)
+{
+   XrdCl::URL url(path);
+   std::string file = url.GetPath();
+   XrdCl::LocationInfo *info = nullptr;
+   XrdCl::FileSystem fs(path);
+
+   std::vector<std::string>  xattrkeys;
+   std::vector<XrdCl::XAttr> xattrvals;
+   xattrkeys.push_back("xrdec.strpver");
+   xattrkeys.push_back("xrdec.filesize");
+
+   XrdCl::Buffer queryArgs(5), *queryResp = nullptr;
+   queryArgs.FromString("role");
+   XrdCl::XRootDStatus st = fs.Query(XrdCl::QueryCode::Config, queryArgs, queryResp);
+   // xrootdfs call this function with individual servers. but we can only do 
+   //        // fs.DeepLocate("*"...) agaist a redirector
+   if (!st.IsOK() || queryResp->ToString() == "server"
+                  || queryResp->ToString() == "server\n")
+   {
+       if (queryResp) delete queryResp;
+       if (!admin->Stat(*buf)) 
+           return -1;
+       else
+       {
+           st = fs.GetXAttr(file, xattrkeys, xattrvals, 0);
+           if (! xattrvals[0].value.empty())
+           {
+               std::stringstream sstream0(xattrvals[0].value);
+               sstream0 >> buf->st_mtime;
+               std::stringstream sstream1(xattrvals[1].value);
+               sstream1 >> buf->st_size;
+               buf->st_blocks = (buf->st_size + 512)/512;
+           }
+           return 0;
+       }
+   }
+   else
+       if (queryResp) delete queryResp;
+
+   st = fs.DeepLocate("*", XrdCl::OpenFlags::None, info );
+   std::unique_ptr<XrdCl::LocationInfo> ptr( info );
+   if( !st.IsOK() ) 
+   {
+       errno = ENOENT;
+       return -1;
+   }
+
+   int found = 0;
+   uint64_t verNumMax = 0;
+   struct stat buf_i;
+   XrdPosixConfig::initStat(&buf_i);
+   for( size_t i = 0; i < info->GetSize(); ++i )
+   {
+       std::string url_i = "root://" + info->At(i).GetAddress() + "/" + file;
+       XrdPosixAdmin *admin_i = new XrdPosixAdmin(url_i.c_str());
+ 
+       if (admin_i->Stat(buf_i)) 
+       {
+           if (! S_ISREG(buf_i.st_mode))
+           {
+               memcpy(buf, &buf_i, sizeof(struct stat));
+               if (admin_i) delete admin_i;
+               return 0;
+           }
+           else
+           {
+               if (verNumMax == 0) memcpy(buf, &buf_i, sizeof(struct stat));
+               found = 1;
+           }
+           XrdCl::FileSystem *fs_i = new XrdCl::FileSystem(info->At( i ).GetAddress());
+   
+           xattrvals.clear();
+           st = fs_i->GetXAttr(file, xattrkeys, xattrvals, 0);
+           if (! xattrvals[0].value.empty())
+           {
+               std::stringstream sstream(xattrvals[0].value);
+               uint64_t verNum;
+               sstream >> verNum;
+               if ( verNum > verNumMax )
+               {
+                   verNumMax = verNum;
+                   memcpy(buf, &buf_i, sizeof(struct stat));
+                   buf->st_mtime = verNumMax;   // assume verNum is mtime
+                   std::stringstream sstream(xattrvals[1].value);
+                   sstream >> buf->st_size;
+                   buf->st_blocks = (buf->st_size + 512)/512;
+               }
+           }
+           if (fs_i) delete fs_i;
+       }
+       if (admin_i) delete admin_i;
+   }
+   if (! found) 
+   {
+       errno = ENOENT;
+       return -1;
+   }
+   return 0;
+}
+   
+/******************************************************************************/
+/*                                E c U n l i n k                             */
+/******************************************************************************/
+int XrdPosixXrootd::EcUnlink(const char *path, XrdPosixAdmin *admin)
+{
+    XrdCl::URL url(path);
+    std::string file = url.GetPath();
+    XrdCl::LocationInfo *info = nullptr;
+    XrdCl::FileSystem fs(path);
+
+    XrdCl::Buffer queryArgs(5), *queryResp = nullptr;
+    queryArgs.FromString("role");
+    XrdCl::XRootDStatus st = fs.Query(XrdCl::QueryCode::Config, queryArgs, queryResp);
+    // xrootdfs call this function with individual servers. but we can only do 
+    // fs.DeepLocate("*"...) agaist a redirector
+    if (!st.IsOK() || queryResp->ToString() == "server"
+                   || queryResp->ToString() == "server\n")
+    {
+        if (queryResp) delete queryResp;
+        return XrdPosixMap::Result(admin->Xrd.Rm(admin->Url.GetPathWithParams()));
+    }
+    else
+        if (queryResp) delete queryResp;
+
+    st = fs.DeepLocate("*", XrdCl::OpenFlags::None, info );
+    std::unique_ptr<XrdCl::LocationInfo> ptr( info );
+    if( !st.IsOK() ) 
+    {
+        errno = ENOENT;
+        return -1;
+    }
+    int rc = -ENOENT;
+    for( size_t i = 0; i < info->GetSize(); ++i )
+    {
+        std::string url_i = "root://" + info->At(i).GetAddress() + "/" + file;
+        XrdPosixAdmin *admin_i = new XrdPosixAdmin(url_i.c_str());
+        int x = XrdPosixMap::Result(admin_i->Xrd.Rm(admin_i->Url.GetPathWithParams()));
+        if (x != -ENOENT && rc != 0)
+            rc = x;
+        if (admin_i) delete admin_i;
+    }
+    return rc;
+}
