@@ -1111,6 +1111,8 @@ ssize_t XrdPssFile::Write(const void *buff, off_t offset, size_t blen)
 
 int XrdPssFile::Fstat(struct stat *buff)
 {
+   EPNAME("fstat");
+
 // If we have a file descriptor then return a stat for it
 //
    if (fd >= 0) return (XrdPosixXrootd::Fstat(fd, buff) ? -errno : XrdOssOK);
@@ -1148,8 +1150,7 @@ int XrdPssFile::Fstat(struct stat *buff)
             if (rpInfo->dstURL) free(rpInfo->dstURL);
             rpInfo->dstURL = strdup(lnkbuff);
             rpInfo->fSize = 1;
-std::cerr<<"Pss_fstat: "<<tident<<" "<<rpInfo->tprPath<<" maps "
-         <<tpcPath<<" -> "<<lnkbuff<<"\n"<<std::flush;
+            DEBUG(tident,rpInfo->tprPath<<" maps "<<tpcPath<<" -> "<<lnkbuff);
           }
       }
 
@@ -1304,8 +1305,11 @@ int XrdPssSys::P2OUT(char *pbuff, int pblen, XrdPssUrlInfo &uInfo)
    if (!(n = P2DST(retc, hBuff, sizeof(hBuff), PolPath, path))) return 0;
    path += n;
 
-// Create the new path
+// Create the new path. If the url already contains a userid then use it
+// instead or our internally generated one. We may need an option for this
+// as it may result in unintended side-effects but for now we do that.
 //
+   if (index(hBuff, '@')) theID= "";
    n = snprintf(pbuff,pblen,"%s%s%s/%s",pname,theID,hBuff,path);
 
 // Make sure the path will fit
