@@ -28,8 +28,8 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <stdio.h>
-#include <string.h>
+#include <cstdio>
+#include <cstring>
 
 #include "XrdRmc/XrdRmc.hh"
 #include "XrdRmc/XrdRmcData.hh"
@@ -404,7 +404,7 @@ int XrdRmcData::Read(char *Buff, long long Offs, int rLen)
 //
    Statistics.Add(Now);
 
-// See if a preread needs to be done. We will only do this if no errors occured
+// See if a preread needs to be done. We will only do this if no errors occurred
 //
    if (doPR && cBuff)
       {EnforceMrSw.UnLock();
