@@ -28,11 +28,12 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <string.h>
+#include <string>
+#include <cstring>
 #include <strings.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <errno.h>
+#include <cerrno>
 
 #include "XrdVersion.hh"
 #include "XrdVersionPlugin.hh"
@@ -64,14 +65,16 @@ class XrdSecProtList
 public:
 
 XrdSecPMask_t    protnum;
+bool             needTLS;
 char             protid[XrdSecPROTOIDSIZE+1];
 char            *protargs;
 XrdSecProtocol  *(*ep)(PROTPARMS);
 XrdSecProtList  *Next;
 
-                XrdSecProtList(const char *pid, const char *parg)
+                XrdSecProtList(const char *pid, const char *parg, bool tls)
+                              : needTLS(tls), ep(0), Next(0)
                       {strncpy(protid, pid, sizeof(protid)-1);
-                       protid[XrdSecPROTOIDSIZE] = '\0'; ep = 0; Next = 0;
+                       protid[XrdSecPROTOIDSIZE] = '\0';
                        protargs = (parg ? strdup(parg): (char *)"");
                       }
                ~XrdSecProtList() {} // ProtList objects never get freed!
@@ -88,6 +91,15 @@ XrdVERSIONINFO(XrdSecGetProtocol,secprot);
 
 XrdVERSIONINFO(XrdSecgetService,secserv);
 
+/******************************************************************************/
+/*                          S t a t i c   I t e m s                           */
+/******************************************************************************/
+  
+namespace
+{
+XrdSysMutex pmMutex;
+}
+
 /******************************************************************************/
 /*                X r d S e c P M a n a g e r   M e t h o d s                 */
 /******************************************************************************/
@@ -144,13 +156,29 @@ XrdSecProtocol *XrdSecPManager::Get(const char       *hname,
    XrdSecProtList *pl;
    XrdSecProtocol *pp;
    XrdOucErrInfo   ei;
-   XrdOucErrInfo  *erp = (eri) ? eri : &ei;
+   XrdOucErrInfo  *erp;
+   char *wp;
    int i;
 
-// We support passing the list of protocols via Url parameter
+// We support passing the list of protocols via Url parameter unless this is
+// a proxy server as the url should be merely passed hrough. If the proxy is
+// not forwarding creds, then we use our error object to prevent security
+// yet from using anything but the proxy's credentials.
+// to become more clever
+//
+   if (isProxy)
+      {wp = 0;
+       if (!fwdCreds) eri = 0;
+      } else {
+       XrdOucEnv *envP;
+       if (!eri || (envP = eri->getEnv()) == 0) wp = 0;
+          else wp = envP->Get("xrd.wantprot");
+      }
+
+// Get the appropriate protocol list as well as the right error object
 //
-   char *wp = (eri && eri->getEnv()) ? eri->getEnv()->Get("xrd.wantprot") : 0;
    const char *wantProt = wp ? (const char *)wp : getenv("XrdSecPROTOCOL");
+   erp = (eri) ? eri : &ei;
 
 // We only scan the buffer once
 //
@@ -195,7 +223,8 @@ XrdSecProtocol *XrdSecPManager::Get(const char       *hname,
              strcat(pcomp, ",");
             }
          if (!wantProt || strstr(compProt, pcomp))
-            {if ((pl = Lookup(pname)) || (pl = ldPO(erp, 'c', pname)))
+            {XrdSysMutexHelper pmHelper(pmMutex);
+             if ((pl = Lookup(pname)) || (pl = ldPO(erp, 'c', pname)))
                 {DEBUG("Using " <<pname <<" protocol, args='"
                        <<(pargs ? pargs : "") <<"'");
                  if ((pp = pl->ep('c', hname, endPoint, pargs, erp)))
@@ -228,6 +257,7 @@ XrdSecProtList *XrdSecPManager::Add(XrdOucErrInfo  *eMsg, const char *pid,
                                     const char *parg)
 {
    XrdSecProtList *plp;
+   bool reqTLS = false;
 
 // Make sure we did not overflow the protocol stack
 //
@@ -236,9 +266,25 @@ XrdSecProtList *XrdSecPManager::Add(XrdOucErrInfo  *eMsg, const char *pid,
        return 0;
       }
 
+// Check if this protocol need TLS
+//
+   if (parg && !strncmp(parg, "TLS:",4))
+      {char pBuff[XrdSecPROTOIDSIZE+2];
+       *pBuff = ' ';
+       strcpy(pBuff+1, pid);  // We know it fits
+       if (!tlsProt) tlsProt = strdup(pBuff);
+          else {std::string tmp(tlsProt);
+                tmp.append(pBuff);
+                free(tlsProt);
+                tlsProt = strdup(tmp.c_str());
+               }
+        parg += 4; // Skip 'TLS:'
+        reqTLS = true;
+       }
+
 // Add this protocol to our protocol stack
 //
-   plp = new XrdSecProtList((char *)pid, parg);
+   plp = new XrdSecProtList((char *)pid, parg, reqTLS);
    plp->ep = ep;
    myMutex.Lock();
    if (Last) {Last->Next = plp; Last = plp;}
