@@ -27,8 +27,8 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <errno.h>
-#include <string.h>
+#include <cerrno>
+#include <cstring>
 
 #include "XrdOuc/XrdOucPList.hh"
 #include "XrdSsi/XrdSsiDir.hh"
