@@ -27,9 +27,9 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <errno.h>
-#include <stdio.h>
-#include <string.h>
+#include <cerrno>
+#include <cstdio>
+#include <cstring>
   
 #include "XrdSsi/XrdSsiResource.hh"
 #include "XrdSsi/XrdSsiRRAgent.hh"
@@ -38,6 +38,10 @@
 #include "XrdSsi/XrdSsiSessReal.hh"
 #include "XrdSsi/XrdSsiTrace.hh"
 #include "XrdSsi/XrdSsiUtils.hh"
+
+#ifndef ENOSR
+#define ENOSR ENOSPC
+#endif
   
 /******************************************************************************/
 /*                     S t a t i c s   &   G l o b a l s                      */
