@@ -40,7 +40,7 @@
 #include <unistd.h>
 #include <strings.h>
 #include <sys/param.h>
-#include <sys/termios.h>
+#include <termios.h>
 #include <sys/uio.h>
 #endif // WIN32
 
