@@ -21,6 +21,7 @@
 #include <openssl/crypto.h>
 #include <openssl/err.h>
 #include <openssl/ssl.h>
+#include <openssl/opensslv.h>
 #include <sys/stat.h>
 
 #include "XrdOuc/XrdOucUtils.hh"
@@ -70,6 +71,9 @@ struct XrdTlsContextImpl
     short                         flushT;
     bool                          crlRunning;
     bool                          flsRunning;
+    time_t                        lastCertModTime = 0;
+    int                           sessionCacheOpts = -1;
+    std::string                   sessionCacheId;
 };
   
 /******************************************************************************/
@@ -118,38 +122,39 @@ do{ctxImpl->crlMutex.ReadLock();
 //
    XrdSysTimer::Snooze(sleepTime);
 
-// Check if this context is still alive. Generally, it never gets deleted.
-//
-   ctxImpl->crlMutex.WriteLock();
-   if (!ctxImpl->owner) break;
+   if (ctxImpl->owner->x509Verify() || ctxImpl->owner->newHostCertificateDetected()) {
+       // Check if this context is still alive. Generally, it never gets deleted.
+       //
+       ctxImpl->crlMutex.WriteLock();
+       if (!ctxImpl->owner) break;
 
-// We clone the original, this will give us the latest crls (i.e. refreshed).
-// We drop the lock while doing so as this may take a long time. This is
-// completely safe to do because we implicitly own the implementation.
-//
-   ctxImpl->crlMutex.UnLock();
-   XrdTlsContext *newctx = ctxImpl->owner->Clone();
+       // We clone the original, this will give us the latest crls (i.e. refreshed).
+       // We drop the lock while doing so as this may take a long time. This is
+       // completely safe to do because we implicitly own the implementation.
+       //
+       ctxImpl->crlMutex.UnLock();
+       XrdTlsContext *newctx = ctxImpl->owner->Clone();
 
-// Verify that the context was properly built
-//
-   if (!newctx || !newctx->isOK())
-      {XrdTls::Emsg("CrlRefresh:","Refresh of context failed!!!",false);
-       continue;
-      }
+       // Verify that the context was properly built
+       //
+       if (!newctx || !newctx->isOK())
+       {XrdTls::Emsg("CrlRefresh:","Refresh of context failed!!!",false);
+           continue;
+       }
 
-// OK, set the new context to be used next time Session() is called.
-//
-   ctxImpl->crlMutex.WriteLock();
-   doreplace = (ctxImpl->ctxnew != 0);
-   if (doreplace) delete ctxImpl->ctxnew;
-   ctxImpl->ctxnew = newctx;
-   ctxImpl->crlMutex.UnLock();
-
-// Do some debugging
-//
-   if (doreplace) {DBG_CTX("CRL refresh created replacement x509 store.");}
-      else {DBG_CTX("CRL refresh created new x509 store.");}
+       // OK, set the new context to be used next time Session() is called.
+       //
+       ctxImpl->crlMutex.WriteLock();
+       doreplace = (ctxImpl->ctxnew != 0);
+       if (doreplace) delete ctxImpl->ctxnew;
+       ctxImpl->ctxnew = newctx;
+       ctxImpl->crlMutex.UnLock();
 
+       // Do some debugging
+       //
+       if (doreplace) {DBG_CTX("CRL refresh created replacement x509 store.");}
+       else {DBG_CTX("CRL refresh created new x509 store.");}
+   }
   } while(true);
 
 // If we are here the context that started us has gone away and we are done
@@ -427,7 +432,7 @@ void Fatal(std::string *eMsg, const char *msg, bool sslmsg=false)
 
 const char *GetTlsMethod(const SSL_METHOD *&meth)
 {
-#ifdef HAVE_TLS
+#if OPENSSL_VERSION_NUMBER > 0x1010000fL /* v1.1.0 */
   meth = TLS_method();
 #else
   meth = SSLv23_method();
@@ -616,13 +621,19 @@ XrdTlsContext::XrdTlsContext(const char *cert,  const char *key,
 
 // Copy parameters to out parm structure.
 //
-   if (cert)   pImpl->Parm.cert   = cert;
+   if (cert)   {
+       pImpl->Parm.cert   = cert;
+       //This call should not fail as a stat is already performed in the call of VerPaths() above
+       XrdOucUtils::getModificationTime(pImpl->Parm.cert.c_str(),pImpl->lastCertModTime);
+   }
    if (key)    pImpl->Parm.pkey   = key;
    if (caDir)  pImpl->Parm.cadir  = caDir;
    if (caFile) pImpl->Parm.cafile = caFile;
    pImpl->Parm.opts   = opts;
-   if (opts & crlRF)
-      pImpl->Parm.crlRT = static_cast<int>((opts & crlRF)>>crlRS);
+   if (opts & crlRF) {
+       // What we store in crlRF is the time in minutes, convert it back to seconds
+       pImpl->Parm.crlRT = static_cast<int>((opts & crlRF) >> crlRS) * 60;
+   }
 
 // Get the correct method to use for TLS and check if successful create a
 // server context that uses the method.
@@ -718,8 +729,11 @@ XrdTlsContext::XrdTlsContext(const char *cert,  const char *key,
    if (SSL_CTX_check_private_key(pImpl->ctx) != 1 )
       FATAL_SSL("Unable to create TLS context; cert-key mismatch.");
 
-// All went well, so keep the context.
+// All went well, start the CRL refresh thread and keep the context.
 //
+   if(opts & rfCRL) {
+       SetCrlRefresh();
+   }
    ctx_tracker.Keep();
 }
   
@@ -743,7 +757,7 @@ XrdTlsContext::~XrdTlsContext()
 /*                                 C l o n e                                  */
 /******************************************************************************/
 
-XrdTlsContext *XrdTlsContext::Clone(bool full)
+XrdTlsContext *XrdTlsContext::Clone(bool full,bool startCRLRefresh)
 {
   XrdTlsContext::CTX_Params &my = pImpl->Parm;
   const char *cert = (my.cert.size()   ? my.cert.c_str()   : 0);
@@ -757,11 +771,23 @@ XrdTlsContext *XrdTlsContext::Clone(bool full)
 
 // Cloning simply means getting a object with the old parameters.
 //
-   XrdTlsContext *xtc = new XrdTlsContext(cert, pkey, caD, caF, my.opts);
+   uint64_t myOpts = my.opts;
+    if(startCRLRefresh){
+        myOpts |= XrdTlsContext::rfCRL;
+    } else {
+        myOpts &= ~XrdTlsContext::rfCRL;
+    }
+   XrdTlsContext *xtc = new XrdTlsContext(cert, pkey, caD, caF, myOpts);
 
 // Verify that the context was built
 //
-   if (xtc->isOK()) return xtc;
+   if (xtc->isOK()) {
+       if(pImpl->sessionCacheOpts != -1){
+           //A SessionCache() call was done for the current context, so apply it for this new cloned context
+           xtc->SessionCache(pImpl->sessionCacheOpts,pImpl->sessionCacheId.c_str(),pImpl->sessionCacheId.size());
+       }
+       return xtc;
+   }
 
 // We failed, cleanup.
 //
@@ -864,12 +890,27 @@ void *XrdTlsContext::Session()
 // magic. For OpenSSL < 1.1, Two stores need to be set with the "set1" variant.
 // Newer version only require SSL_CTX_set1_cert_store() to be used.
 //
-   X509_STORE *newX509 = SSL_CTX_get_cert_store(pImpl->ctx);
+   //We have a new context generated by Refresh, so we must use it.
+   XrdTlsContext * ctxnew = pImpl->ctxnew;
 
 #if OPENSSL_VERSION_NUMBER < 0x10101000L
+   /*X509_STORE *newX509 = SSL_CTX_get_cert_store(ctxnew->pImpl->ctx);
    SSL_CTX_set1_verify_cert_store(pImpl->ctx, newX509);
-   SSL_CTX_set1_chain_cert_store(pImpl->ctx, newX509);
+   SSL_CTX_set1_chain_cert_store(pImpl->ctx, newX509);*/
+   //The above two macros actually do not replace the certificate that has
+   //to be used for that SSL session, so we will create the session with the SSL_CTX * of
+   //the TlsContext created by Refresh()
+   //First, free the current SSL_CTX, if it is used by any transfer, it will just decrease
+   //the reference counter of it. There is therefore no risk of double free...
+   SSL_CTX_free(pImpl->ctx);
+   pImpl->ctx = ctxnew->pImpl->ctx;
+   //In the destructor of XrdTlsContextImpl, SSL_CTX_Free() is
+   //called if ctx is != 0. As this new ctx is used by the session
+   //we just created, we don't want that to happen. We therefore set it to 0.
+   //The SSL_free called on the session will cleanup the context for us.
+   ctxnew->pImpl->ctx = 0;
 #else
+   X509_STORE *newX509 = SSL_CTX_get_cert_store(ctxnew->pImpl->ctx);
    SSL_CTX_set1_cert_store(pImpl->ctx, newX509);
 #endif
 
@@ -907,6 +948,9 @@ int XrdTlsContext::SessionCache(int opts, const char *id, int idlen)
    long sslopt = 0;
    int flushT = opts & scFMax;
 
+   pImpl->sessionCacheOpts = opts;
+   pImpl->sessionCacheId = id;
+
 // If initialization failed there is nothing to do
 //
    if (pImpl->ctx == 0) return 0;
@@ -989,31 +1033,13 @@ bool XrdTlsContext::SetCrlRefresh(int refsec)
    pthread_t tid;
    int       rc;
 
-// If the value is zero, then the caller want to stop refreshing.
+// If it's negative or equal to 0, use the current setting
 //
-   if (!refsec)
-      {pImpl->crlMutex.WriteLock();
-       pImpl->Parm.crlRT = 0;
-       pImpl->crlMutex.UnLock();
-       return true;
-      }
-
-// If it's negative, use the curren setting
-//
-   if (refsec < 0)
+   if (refsec <= 0)
       {pImpl->crlMutex.WriteLock();
        refsec = pImpl->Parm.crlRT;
        pImpl->crlMutex.UnLock();
-       if (!refsec) refsec = 8*60*60;
-      }
-
-
-// Check if there is anything that is refreshable
-//
-   if (!x509Verify())
-      {XrdTls::Emsg("CrlRefresh:",
-                    "No cert information exists to refresh!", false);
-       return false;
+       if (!refsec) refsec = XrdTlsContext::DEFAULT_CRL_REF_INT_SEC;
       }
 
 // Make sure this is at least 60 seconds between refreshes
@@ -1060,3 +1086,20 @@ bool XrdTlsContext::x509Verify()
 {
    return !(pImpl->Parm.cadir.empty()) || !(pImpl->Parm.cafile.empty());
 }
+
+bool XrdTlsContext::newHostCertificateDetected() {
+    const std::string certPath = pImpl->Parm.cert;
+    if(certPath.empty()) {
+        //No certificate provided, should not happen though
+        return false;
+    }
+    time_t modificationTime;
+    if(!XrdOucUtils::getModificationTime(certPath.c_str(),modificationTime)){
+        if (pImpl->lastCertModTime != modificationTime) {
+            //The certificate file has changed
+            pImpl->lastCertModTime = modificationTime;
+            return true;
+        }
+    }
+    return false;
+}
\ No newline at end of file
