@@ -32,6 +32,7 @@
 #include "XrdNet/XrdNetAddrInfo.hh"
 #include "XrdSys/XrdSysE2T.hh"
 #include "XrdSys/XrdSysPthread.hh"
+#include "XrdTls/XrdTls.hh"
 #include "XrdTls/XrdTlsContext.hh"
 #include "XrdTls/XrdTlsNotary.hh"
 #include "XrdTls/XrdTlsPeerCerts.hh"
@@ -95,7 +96,6 @@ XrdTlsSocketImpl *theImpl;
 
 namespace XrdTlsGlobal
 {
-extern XrdTls::msgCB_t msgCB;
 extern XrdSysTrace SysTrace;
 }
 
@@ -216,8 +216,15 @@ do{if ((rc = SSL_accept( pImpl->ssl )) > 0)
 
    // Check why we did not succeed. We may be able to recover.
    //
-   if (ssler != SSL_ERROR_WANT_READ && ssler != SSL_ERROR_WANT_WRITE)
-      {aOK = false; break;}
+   if (ssler != SSL_ERROR_WANT_READ && ssler != SSL_ERROR_WANT_WRITE) {
+       if(ssler == SSL_ERROR_SSL){
+           //In the case the accept does have an error related to OpenSSL,
+           //shutdown the TLSSocket in case the link associated to that connection
+           //is re-used
+           Shutdown();
+       }
+       aOK = false; break;
+   }
 
    if (pImpl->hsNoBlock) return XrdTls::ssl2RC(ssler);
 
@@ -743,7 +750,7 @@ void XrdTlsSocket::Shutdown(XrdTlsSocket::SDType sdType)
                 std::string eMsg = Err2Text(rc);
                 snprintf(msgBuff, sizeof(msgBuff),
                         "FD %d TLS shutdown failed; %s.\n",pImpl->sFD,eMsg.c_str());
-                XrdTlsGlobal::msgCB(pImpl->traceID, msgBuff, false);
+                XrdTls::Emsg(pImpl->traceID, msgBuff, true);
                 break;
                }
            }
