@@ -12,6 +12,7 @@
 
 #include <curl/curl.h>
 
+#include <algorithm>
 #include <sstream>
 #include <stdexcept>
 
@@ -33,7 +34,10 @@ public:
     MultiCurlHandler(std::vector<State*> &states, XrdSysError &log) :
         m_handle(curl_multi_init()),
         m_states(states),
-        m_log(log)
+        m_log(log),
+        m_bytes_transferred(0),
+        m_error_code(0),
+        m_status_code(0)
     {
         if (m_handle == NULL) {
             throw CurlHandlerSetupError("Failed to initialize a libcurl multi-handle");
@@ -54,12 +58,6 @@ public:
              it != m_active_handles.end();
              it++) {
             curl_multi_remove_handle(m_handle, *it);
-            curl_easy_cleanup(*it);
-        }
-        for (std::vector<CURL *>::const_iterator it = m_avail_handles.begin();
-             it != m_avail_handles.end();
-             it++) {
-            curl_easy_cleanup(*it);
         }
         curl_multi_cleanup(m_handle);
     }
@@ -80,6 +78,17 @@ public:
              state_iter != m_states.end();
              state_iter++) {
             if (curl == (*state_iter)->GetHandle()) {
+                m_bytes_transferred += (*state_iter)->BytesTransferred();
+                int error_code = (*state_iter)->GetErrorCode();
+                if (error_code && !m_error_code) {
+                    m_error_code = error_code;
+                    m_error_message = (*state_iter)->GetErrorMessage();
+                }
+                int status_code = (*state_iter)->GetStatusCode();
+                if (status_code >= 400 && !m_status_code) {
+                    m_status_code = status_code;
+                    m_error_message = (*state_iter)->GetErrorMessage();
+                }
                 (*state_iter)->ResetAfterRequest();
                 break;
             }
@@ -118,6 +127,42 @@ public:
         return current_offset;
     }
 
+    int Flush() {
+        int last_error = 0;
+        for (std::vector<State*>::iterator state_it = m_states.begin();
+             state_it != m_states.end();
+             state_it++)
+        {
+            int error = (*state_it)->Flush();
+            if (error) {last_error = error;}
+        }
+        return last_error;
+    }
+
+    off_t BytesTransferred() const {
+        return m_bytes_transferred;
+    }
+
+    int GetStatusCode() const {
+        return m_status_code;
+    }
+
+    int GetErrorCode() const {
+        return m_error_code;
+    }
+
+    void SetErrorCode(int error_code) {
+        m_error_code = error_code;
+    }
+
+    std::string GetErrorMessage() const {
+        return m_error_message;
+    }
+
+    void SetErrorMessage(const std::string &error_msg) {
+        m_error_message = error_msg;
+    }
+
 private:
 
     bool StartTransfer(off_t offset, size_t size) {
@@ -203,12 +248,17 @@ private:
     std::vector<CURL *> m_active_handles;
     std::vector<State*> &m_states;
     XrdSysError         &m_log;
+    off_t                m_bytes_transferred;
+    int                  m_error_code;
+    int                  m_status_code;
+    std::string          m_error_message;
 };
 }
 
 
 int TPCHandler::RunCurlWithStreamsImpl(XrdHttpExtReq &req, State &state,
-    size_t streams, std::vector<State*> handles, TPCLogRecord &rec)
+    size_t streams, std::vector<State*> &handles,
+    std::vector<ManagedCurlHandle> &curl_handles, TPCLogRecord &rec)
 {
     int result;
     bool success;
@@ -228,6 +278,7 @@ int TPCHandler::RunCurlWithStreamsImpl(XrdHttpExtReq &req, State &state,
     handles[0]->Move(state);
     for (size_t idx = 1; idx < concurrency; idx++) {
         handles.push_back(handles[0]->Duplicate());
+        curl_handles.emplace_back(handles.back()->GetHandle());
     }
 
     // Create the multi-handle and add in the current transfer to it.
@@ -257,17 +308,33 @@ int TPCHandler::RunCurlWithStreamsImpl(XrdHttpExtReq &req, State &state,
     // Transfer loop: use curl to actually run the transfer, but periodically
     // interrupt things to send back performance updates to the client.
     time_t last_marker = 0;
+    // Track the time since the transfer last made progress
+    off_t last_advance_bytes = 0;
+    time_t last_advance_time = time(NULL);
+    time_t transfer_start = last_advance_time;
     CURLcode res = static_cast<CURLcode>(-1);
-    CURLMcode mres;
+    CURLMcode mres = CURLM_OK;
     do {
         time_t now = time(NULL);
         time_t next_marker = last_marker + m_marker_period;
         if (now >= next_marker) {
+            if (current_offset > last_advance_bytes) {
+                last_advance_bytes = current_offset;
+                last_advance_time = now;
+            }
             if (SendPerfMarker(req, rec, handles, current_offset)) {
                 logTransferEvent(LogMask::Error, rec, "PERFMARKER_FAIL",
                     "Failed to send a perf marker to the TPC client");
                 return -1;
             }
+            int timeout = (transfer_start == last_advance_time) ? m_first_timeout : m_timeout;
+            if (now > last_advance_time + timeout) {
+                mch.SetErrorCode(10);
+                std::stringstream ss;
+                ss << "Transfer failed because no bytes have been received in " << timeout << " seconds.";
+                mch.SetErrorMessage(ss.str());
+                break;
+            }
             last_marker = now;
         }
 
@@ -287,8 +354,8 @@ int TPCHandler::RunCurlWithStreamsImpl(XrdHttpExtReq &req, State &state,
             msg = curl_multi_info_read(multi_handle, &msgq);
             if (msg && (msg->msg == CURLMSG_DONE)) {
                 CURL *easy_handle = msg->easy_handle;
-                mch.FinishCurlXfer(easy_handle);
                 res = msg->data.result;
+                mch.FinishCurlXfer(easy_handle);
                 // If any requests fail, cut off the entire transfer.
                 if (res != CURLE_OK) {
                     break;
@@ -359,22 +426,38 @@ int TPCHandler::RunCurlWithStreamsImpl(XrdHttpExtReq &req, State &state,
         }
     } while (msg);
 
-    if (res == static_cast<CURLcode>(-1)) { // No transfers returned?!?
+    if (!state.GetErrorCode() && res == static_cast<CURLcode>(-1)) { // No transfers returned?!?
         logTransferEvent(LogMask::Error, rec, "MULTISTREAM_ERROR",
             "Internal state error in libcurl");
         throw std::runtime_error("Internal state error in libcurl");
     }
 
-    rec.bytes_transferred = state.BytesTransferred();
-    rec.tpc_status = state.GetStatusCode();
+    mch.Flush();
+
+    rec.bytes_transferred = mch.BytesTransferred();
+    rec.tpc_status = mch.GetStatusCode();
 
     // Generate the final response back to the client.
     std::stringstream ss;
     success = false;
-    if (state.GetStatusCode() >= 400) {
-        ss << "failure: Remote side failed with status code " << state.GetStatusCode();
-        rec.status = state.GetStatusCode();
+    if (mch.GetStatusCode() >= 400) {
+        std::string err = mch.GetErrorMessage();
+        std::stringstream ss2;
+        ss2 << "Remote side failed with status code " << mch.GetStatusCode();
+        if (!err.empty()) {
+            std::replace(err.begin(), err.end(), '\n', ' ');
+            ss2 << "; error message: \"" << err << "\"";
+        }
         logTransferEvent(LogMask::Error, rec, "MULTISTREAM_FAIL", ss.str());
+        ss << "failure: " << ss2.str();
+    } else if (mch.GetErrorCode()) {
+        std::string err = mch.GetErrorMessage();
+        if (err.empty()) {err = "(no error message provided)";}
+        else {std::replace(err.begin(), err.end(), '\n', ' ');}
+        std::stringstream ss2;
+        ss2 << "Error when interacting with local filesystem: " << err;
+        logTransferEvent(LogMask::Error, rec, "MULTISTREAM_FAIL", ss2.str());
+        ss << "failure: " << ss2.str();
     } else if (res != CURLE_OK) {
         std::stringstream ss2;
         ss2 << "Request failed when processing: " << curl_easy_strerror(res);
@@ -401,6 +484,7 @@ int TPCHandler::RunCurlWithStreamsImpl(XrdHttpExtReq &req, State &state,
         return retval;
     } else if (success) {
         logTransferEvent(LogMask::Info, rec, "TRANSFER_SUCCESS");
+        rec.status = 0;
     }
     return req.ChunkResp(NULL, 0);
 }
@@ -409,9 +493,10 @@ int TPCHandler::RunCurlWithStreamsImpl(XrdHttpExtReq &req, State &state,
 int TPCHandler::RunCurlWithStreams(XrdHttpExtReq &req, State &state,
     size_t streams, TPCLogRecord &rec)
 {
+    std::vector<ManagedCurlHandle> curl_handles;
     std::vector<State*> handles;
     try {
-        int retval = RunCurlWithStreamsImpl(req, state, streams, handles, rec);
+        int retval = RunCurlWithStreamsImpl(req, state, streams, handles, curl_handles, rec);
         for (std::vector<State*>::iterator state_iter = handles.begin();
              state_iter != handles.end();
              state_iter++) {
