@@ -28,9 +28,10 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <errno.h>
-#include <stdio.h>
-#include <time.h>
+#include <cerrno>
+#include <cstdio>
+#include <ctime>
+#include <limits.h>
 #include <sys/uio.h>
 #include "Xrd/XrdLink.hh"
 #include "Xrd/XrdScheduler.hh"
@@ -373,7 +374,7 @@ bool XrdXrootdAioTask::Validate(XrdXrootdAioBuff* aioP)
 //
    if (isDone) return false;
 
-// Step 2: Check if an error occured as this will terminate the request even
+// Step 2: Check if an error occurred as this will terminate the request even
 //         if we have not sent all of the data.
 //
    if (aioP->Result < 0)
@@ -446,7 +447,8 @@ bool XrdXrootdAioTask::Wait4Buff(int maxWait)
 
 // Calculate wait time and when we should produce a message
 //
-   if (maxWait <= 0)  maxWait = XrdXrootdProtocol::as_timeout;
+   if (maxWait <= 0)  maxWait = (XrdXrootdProtocol::as_timeout ?
+                                 XrdXrootdProtocol::as_timeout : INT_MAX);
    aioWait = (maxWait > msgInterval ? msgInterval : maxWait);
 
 // Wait for a buffer to arrive.
