@@ -28,9 +28,9 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <errno.h>
-#include <stdio.h>
-#include <string.h>
+#include <cerrno>
+#include <cstdio>
+#include <cstring>
 #include <netinet/in.h>
 #include <sys/uio.h>
 
@@ -93,7 +93,7 @@ int                        Result;
 /*                               G l o b a l s                                */
 /******************************************************************************/
 
-extern XrdOucTrace       *XrdXrootdTrace;
+extern XrdSysTrace        XrdXrootdTrace;
 
 namespace
 {
