@@ -31,8 +31,8 @@
 #include <dirent.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include <stdio.h>
-#include <stdlib.h>
+#include <cstdio>
+#include <cstdlib>
 #include <strings.h>
 #include <sys/param.h>
 #include <sys/types.h>
@@ -59,7 +59,7 @@
 /******************************************************************************/
 
 #ifndef NODEBUG  
-extern XrdOucTrace     *XrdXrootdTrace;
+extern XrdSysTrace      XrdXrootdTrace;
 #endif
 
        XrdScheduler    *XrdXrootdPrepare::SchedP;
