@@ -26,6 +26,8 @@
 /* be used to endorse or promote products derived from this software without  */
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
+
+#include <poll.h>
  
 #include "XrdVersion.hh"
 
@@ -33,21 +35,25 @@
 
 #include "Xrd/XrdBuffer.hh"
 #include "Xrd/XrdLink.hh"
-#include "XrdOuc/XrdOucStream.hh"
+#include "XrdNet/XrdNetIF.hh"
 #include "XrdOuc/XrdOucEnv.hh"
 #include "XrdOuc/XrdOucUtils.hh"
+#include "XrdOuc/XrdOucStream.hh"
+#include "XrdOuc/XrdOucString.hh"
+#include "XrdOuc/XrdOucUtils.hh"
 #include "XrdSec/XrdSecProtect.hh"
 #include "XrdSfs/XrdSfsFlags.hh"
 #include "XrdSfs/XrdSfsInterface.hh"
 #include "XrdSys/XrdSysAtomics.hh"
 #include "XrdSys/XrdSysTimer.hh"
-#include "XrdXrootd/XrdXrootdAio.hh"
+#include "XrdTls/XrdTls.hh"
 #include "XrdXrootd/XrdXrootdFile.hh"
 #include "XrdXrootd/XrdXrootdFileLock.hh"
 #include "XrdXrootd/XrdXrootdFileLock1.hh"
 #include "XrdXrootd/XrdXrootdMonFile.hh"
 #include "XrdXrootd/XrdXrootdMonitor.hh"
 #include "XrdXrootd/XrdXrootdPio.hh"
+#include "XrdXrootd/XrdXrootdPgwCtl.hh"
 #include "XrdXrootd/XrdXrootdProtocol.hh"
 #include "XrdXrootd/XrdXrootdStats.hh"
 #include "XrdXrootd/XrdXrootdTrace.hh"
@@ -57,8 +63,20 @@
 /*                               G l o b a l s                                */
 /******************************************************************************/
 
-XrdOucTrace          *XrdXrootdTrace;
+namespace XrdXrootd
+{
+XrdSysError           eLog(0, "Xrootd");
+XrdBuffManager       *BPool;
+XrdScheduler         *Sched;
+XrdXrootdStats       *SI;
+}
 
+XrdSysTrace           XrdXrootdTrace("Xrootd");
+
+/******************************************************************************/
+/*                        S t a t i c   M e m b e r s                         */
+/******************************************************************************/
+  
 XrdXrootdXPath        XrdXrootdProtocol::RPList;
 XrdXrootdXPath        XrdXrootdProtocol::RQList;
 XrdXrootdXPath        XrdXrootdProtocol::XPList;
@@ -70,7 +88,8 @@ XrdSecProtector      *XrdXrootdProtocol::DHS      = 0;
 XrdTlsContext        *XrdXrootdProtocol::tlsCtx   = 0;
 XrdScheduler         *XrdXrootdProtocol::Sched;
 XrdBuffManager       *XrdXrootdProtocol::BPool;
-XrdSysError           XrdXrootdProtocol::eDest(0, "Xrootd");
+XrdSysError          &XrdXrootdProtocol::eDest = XrdXrootd::eLog;
+XrdNetPMark          *XrdXrootdProtocol::PMark    = 0;
 XrdXrootdStats       *XrdXrootdProtocol::SI;
 XrdXrootdJob         *XrdXrootdProtocol::JobCKS   = 0;
 char                 *XrdXrootdProtocol::JobCKT   = 0;
@@ -97,18 +116,21 @@ int                   XrdXrootdProtocol::maxTransz    = 262144; // 256KB
 int                   XrdXrootdProtocol::as_maxperlnk = 8;   // Max ops per link
 int                   XrdXrootdProtocol::as_maxperreq = 8;   // Max ops per request
 int                   XrdXrootdProtocol::as_maxpersrv = 4096;// Max ops per server
-int                   XrdXrootdProtocol::as_segsize   = 131072;
-int                   XrdXrootdProtocol::as_miniosz   = 32768;
+int                   XrdXrootdProtocol::as_seghalf   = 32768;
+int                   XrdXrootdProtocol::as_segsize   = 65536;
+int                   XrdXrootdProtocol::as_miniosz   = 98304;
 #ifdef __solaris__
 int                   XrdXrootdProtocol::as_minsfsz   = 1;
 #else
 int                   XrdXrootdProtocol::as_minsfsz   = 8192;
 #endif
-int                   XrdXrootdProtocol::as_maxstalls = 5;
-int                   XrdXrootdProtocol::as_force     = 0;
-int                   XrdXrootdProtocol::as_noaio     = 0;
-int                   XrdXrootdProtocol::as_nosf      = 0;
-int                   XrdXrootdProtocol::as_syncw     = 0;
+int                   XrdXrootdProtocol::as_maxstalls = 4;
+short                 XrdXrootdProtocol::as_okstutter = 1; // For 64K unit
+short                 XrdXrootdProtocol::as_timeout   = 45;
+bool                  XrdXrootdProtocol::as_force     = false;
+bool                  XrdXrootdProtocol::as_aioOK     = true;
+bool                  XrdXrootdProtocol::as_nosf      = false;
+bool                  XrdXrootdProtocol::as_syncw     = false;
 
 const char           *XrdXrootdProtocol::myInst  = 0;
 const char           *XrdXrootdProtocol::TraceID = "Protocol";
@@ -131,10 +153,15 @@ bool                  XrdXrootdProtocol::PrepareAlt = false;
 bool                  XrdXrootdProtocol::LimitError = true;
 
 struct XrdXrootdProtocol::RD_Table XrdXrootdProtocol::Route[RD_Num];
+struct XrdXrootdProtocol::RC_Table XrdXrootdProtocol::RouteClient;
 int                   XrdXrootdProtocol::OD_Stall = 33;
 bool                  XrdXrootdProtocol::OD_Bypass= false;
 bool                  XrdXrootdProtocol::OD_Redir = false;
 
+bool                  XrdXrootdProtocol::CL_Redir = false;
+
+bool                  XrdXrootdProtocol::isProxy  = false;
+
 int                   XrdXrootdProtocol::usxMaxNsz= kXR_faMaxNlen;
 int                   XrdXrootdProtocol::usxMaxVsz= kXR_faMaxVlen;
 char                 *XrdXrootdProtocol::usxParms = 0;
@@ -142,6 +169,8 @@ char                 *XrdXrootdProtocol::usxParms = 0;
 char                  XrdXrootdProtocol::tlsCap   = 0;
 char                  XrdXrootdProtocol::tlsNot   = 0;
 
+RAtomic_int           XrdXrootdProtocol::srvrAioOps = {0};
+
 /******************************************************************************/
 /*            P r o t o c o l   M a n a g e m e n t   S t a c k s             */
 /******************************************************************************/
@@ -149,7 +178,7 @@ char                  XrdXrootdProtocol::tlsNot   = 0;
 XrdObjectQ<XrdXrootdProtocol>
             XrdXrootdProtocol::ProtStack("ProtStack",
                                        "xroot protocol anchor");
-
+  
 /******************************************************************************/
 /*                       P r o t o c o l   L o a d e r                        */
 /*                        X r d g e t P r o t o c o l                         */
@@ -191,7 +220,7 @@ XrdProtocol *XrdgetProtocol(const char *pname, char *parms,
 /******************************************************************************/
 
 // This function is called early on to determine the port we need to use. The
-// default is ostensibly 1094 but can be overidden; which we allow.
+// default is ostensibly 1094 but can be overridden; which we allow.
 //
 XrdVERSIONINFO(XrdgetProtocolPort,xrootd);
 
@@ -230,7 +259,7 @@ XrdXrootdProtocol::XrdXrootdProtocol()
 {
    Reset();
 }
-
+  
 /******************************************************************************/
 /* protected:                     g e t S I D                                 */
 /******************************************************************************/
@@ -340,10 +369,7 @@ int XrdXrootdProtocol::Process(XrdLink *lp) // We ignore the argument here
 // Check if we are servicing a slow link
 //
    if (Resume)
-      {if (myBlen && (rc = getData("data", myBuff, myBlen)) != 0)
-          {if (rc < 0 && myAioReq) myAioReq->Recycle(-1);
-           return rc;
-          }
+      {if (myBlen && (rc = getData("data", myBuff, myBlen)) != 0) return rc;
           else if ((rc = (*this.*Resume)()) != 0) return rc;
                   else {Resume = 0; return 0;}
       }
@@ -383,7 +409,7 @@ int XrdXrootdProtocol::Process(XrdLink *lp) // We ignore the argument here
 // Read any argument data at this point, except when the request is a write.
 // The argument may have to be segmented and we're not prepared to do that here.
 //
-   if (reqID != kXR_write && Request.header.dlen)
+   if (reqID != kXR_write && reqID != kXR_pgwrite && Request.header.dlen)
       {if (!argp || Request.header.dlen+1 > argp->bsize)
           {if (argp) BPool->Release(argp);
            if (!(argp = BPool->Obtain(Request.header.dlen+1)))
@@ -449,7 +475,9 @@ int XrdXrootdProtocol::Process2()
 
 // Help the compiler, select the the high activity requests (the ones with
 // file handles) in a separate switch statement. A special case exists for
-// sync() which return with a callback, so handle it here.
+// sync() which return with a callback, so handle it here. Note that stat(fh)
+// normally never does a callback but historically we allowed it to do so.
+// We maintain that capability even when it's likely never used.
 //
    switch(Request.header.requestid)   // First, the ones with file handles
          {case kXR_read:     return do_Read();
@@ -462,6 +490,11 @@ int XrdXrootdProtocol::Process2()
                              return do_Sync();
           case kXR_close:    ReqID.setID(Request.header.streamid);
                              return do_Close();
+          case kXR_stat:     if (!Request.header.dlen)
+                                {ReqID.setID(Request.header.streamid);
+                                 return do_Stat();
+                                }
+                             break;
           case kXR_truncate: ReqID.setID(Request.header.streamid);
                              if (!Request.header.dlen) return do_Truncate();
                              break;
@@ -482,11 +515,14 @@ int XrdXrootdProtocol::Process2()
 // Force authentication at this point, if need be
 //
    if (Status & XRD_NEED_AUTH)
-      {if (Request.header.requestid == kXR_auth) return do_Auth();
+      {int rc;
+       if (Request.header.requestid == kXR_auth) rc = do_Auth();
           else {Response.Send(kXR_InvalidRequest,
                               "Invalid request; user not authenticated");
-                return -1;
+                rc = -1;
                }
+       if (tlsCtx) XrdTls::ClearErrorQueue();
+       return rc;
       }
 
 // Construct request ID as the following functions are async eligible
@@ -496,8 +532,7 @@ int XrdXrootdProtocol::Process2()
 // Process items that don't need arguments but may have them
 //
    switch(Request.header.requestid)
-         {case kXR_stat:      return do_Stat();
-          case kXR_endsess:   return do_Endsess();
+         {case kXR_endsess:   return do_Endsess();
           default:            break;
          }
 
@@ -508,6 +543,29 @@ int XrdXrootdProtocol::Process2()
        return 0;
       }
 
+// All of the subsequent requests can be redirected and are subject to
+// prefunctory redirection which we check here.
+//
+   if (CL_Redir && !Link->hasBridge())
+      {bool doRdr = false;
+       if (Link->AddrInfo()->isPrivate()) rdType = 1;
+            if (RouteClient.pvtIP && rdType) doRdr = true;
+       else if (RouteClient.lclDom && XrdNetIF::InDomain( Link->AddrInfo()))
+               doRdr = true;
+       else if (RouteClient.DomCnt)
+               {XrdOucString hName = Link->Host();
+                for (int i = 0; i < RouteClient.DomCnt; i++)
+                    {if (hName.endswith(RouteClient.Domain[i]))
+                        {doRdr = true; break;}
+                    }
+               }
+       if (doRdr)
+          {Response.Send(kXR_redirect,Route[RD_client].Port[rdType],
+                                      Route[RD_client].Host[rdType]);
+           return -1;
+          }
+      }
+
 // Process items that keep own statistics
 //
    switch(Request.header.requestid)
@@ -534,6 +592,7 @@ int XrdXrootdProtocol::Process2()
           case kXR_rm:        return do_Rm();
           case kXR_rmdir:     return do_Rmdir();
           case kXR_set:       return do_Set();
+          case kXR_stat:      return do_Stat();
           case kXR_statx:     return do_Statx();
           case kXR_truncate:  return do_Truncate();
           default:            break;
@@ -598,6 +657,7 @@ void XrdXrootdProtocol::Recycle(XrdLink *lp, int csec, const char *reason)
 {
    char *sfxp, ctbuff[24], buff[128], Flags = (reason ? XROOTD_MON_FORCED : 0);
    const char *What;
+   XrdSysMutexHelper recycleHelper(unbindMutex);
 
 // Check for disconnect or unbind
 //
@@ -616,19 +676,38 @@ void XrdXrootdProtocol::Recycle(XrdLink *lp, int csec, const char *reason)
        eDest.Log(SYS_LOG_02, "Xeq", lp->ID, (char *)What, sfxp);
       }
 
-// If this is a bound stream then we cannot release the resources until
-// the main stream closes this stream (i.e., lp == 0). On the other hand, the
-// main stream will not be trying to do this if we are still tagged as active.
-// So, we need to redrive the main stream to complete the full shutdown.
+// Handle any waiting read on this link. This is a tricky proposition because
+// we don't know if the thread is waiting to run or not. However, we will
+// indicate that gdFail was already called and should the thread run, it will
+// promptly exit should it ever run again. That way, we handle the cleanup.
 //
-   if (Status == XRD_BOUNDPATH && Stream[0])
-      {Stream[0]->streamMutex.Lock();
-       isDead = true;
+   if (gdCtl.linkWait.fetch_or(GetDataCtl::Terminate) == GetDataCtl::Active
+   && (gdCtl.Status == GetDataCtl::inData
+   ||  gdCtl.Status == GetDataCtl::inDataIov)) gdCtl.CallBack->gdFail();
+
+// If this is a bound stream then we cannot release the resources until
+// all activity ceases on this stream (i.e., lp == 0). This is only relevant for
+// writes that read from the link. if we are still tagged as active and not
+// waiting for link activity then ask to be signalled once activity stops.
+// Otherwise, redrive the parallel I/O so that it cleans up.
+//
+   if (lp && Status == XRD_BOUNDPATH)
+      {streamMutex.Lock();
+       isDead = isNOP = true;
        if (isActive)
-          {isActive = false;
-           Stream[0]->Link->setRef(-1);
-          }
-       Stream[0]->streamMutex.UnLock();
+          {if (isLinkWT)
+              {streamMutex.UnLock();
+               do_OffloadIO();
+              } else {
+               while(isActive)
+                    {XrdSysCondVar2 aioDone(streamMutex);
+                     endNote = &aioDone;
+                     aioDone.Wait();
+                     endNote = 0;
+                    }
+               streamMutex.UnLock();
+              }
+          } else streamMutex.UnLock();
        if (lp) return;  // Async close
       }
 
@@ -841,6 +920,22 @@ void XrdXrootdProtocol::Reclaim(XrdSfsXioHandle h)
    if (h) BPool->Release((XrdBuffer *)h);
 }
   
+/******************************************************************************/
+/*                             S t r e a m N O P                              */
+/******************************************************************************/
+
+void XrdXrootdProtocol::StreamNOP()
+{
+
+// Mark this stream as not operation if it is not the control strea,
+//
+   if (PathID)
+      {streamMutex.Lock();
+       isNOP = true;
+       streamMutex.UnLock();
+      }
+}
+  
 /******************************************************************************/
 /*                                  S w a p                                   */
 /******************************************************************************/
@@ -866,6 +961,36 @@ XrdSfsXioHandle XrdXrootdProtocol::Swap(const char *buff, XrdSfsXioHandle h)
    return 0;
 }
   
+/******************************************************************************/
+/*                          V e r i f y S t r e a m                           */
+/******************************************************************************/
+
+XrdXrootdProtocol *XrdXrootdProtocol::VerifyStream(int &rc, int pID, bool lok)
+{
+   XrdXrootdProtocol *pp;
+
+// Verify that the path actually exists
+//
+   if (pID >= maxStreams || !(pp = Stream[pID]))
+      {rc = Response.Send(kXR_ArgInvalid, "invalid path ID");
+       return 0;
+      }
+
+// Verify that this path is still functional
+//
+   pp->streamMutex.Lock();
+   if (pp->isNOP)
+      {pp->streamMutex.UnLock();
+       rc = Response.Send(kXR_ArgInvalid, "path ID is not operational");
+       return 0;
+      }
+
+// All done!
+//
+   if (!lok) pp->streamMutex.UnLock();
+   return pp;
+}
+  
 /******************************************************************************/
 /*                       P r i v a t e   M e t h o d s                        */
 /******************************************************************************/
@@ -915,6 +1040,29 @@ void XrdXrootdProtocol::Cleanup()
    XrdXrootdPio *pioP;
    int i;
 
+// Handle parallel stream cleanup. The session stream cannot be closed if
+// there is any queued activity on subordinate streams. A subordinate
+// can either be closed from the session stream or asynchronously only if
+// it is active. Which means they could be running while we are running.
+// So, we first call close() which should trigger a recycle quiesce. Upon
+// return we can actually recycle the object.
+//
+   if (Status != XRD_BOUNDPATH)
+      {streamMutex.Lock();
+       for (i = 1; i < maxStreams; i++)
+           if (Stream[i])
+              {Stream[i]->Stream[0] = 0;
+               if (!Stream[i]->isDead) Stream[i]->Link->Close();
+               Stream[i]->Recycle(0, 0, 0);
+               Stream[i] = 0;
+              }
+       streamMutex.UnLock();
+      }
+
+// Handle packet parking (needs to be done before deleting other stuff)
+//
+   if (pmHandle) delete pmHandle;
+
 // Release any internal monitoring information
 //
    if (Entity.moninfo) {free(Entity.moninfo); Entity.moninfo = 0;}
@@ -927,6 +1075,13 @@ void XrdXrootdProtocol::Cleanup()
 //
    if (Status != XRD_BOUNDPATH) osFS->Disc(Client);
 
+// Handle parallel I/O appendages. We need to do this first as these have
+// referenced open files and we need to deref them before we cleanup the ftab.
+//
+   while((pioP = pioFirst))
+        {pioP->IO.File->Ref(-1); pioFirst = pioP->Next; pioP->Recycle();}
+   while((pioP = pioFree ))     {pioFree  = pioP->Next; pioP->Recycle();}
+
 // Delete the FTab if we have it
 //
    if (FTab)
@@ -934,23 +1089,6 @@ void XrdXrootdProtocol::Cleanup()
        FTab = 0;
       }
 
-// Handle parallel stream cleanup. The session stream cannot be closed if
-// there is any queued activity on subordinate streams. A subordinate
-// can either be closed from the session stream or asynchronously only if
-// it is active. Which means they could be running while we are running.
-//
-   if (isBound && Status != XRD_BOUNDPATH)
-      {streamMutex.Lock();
-       for (i = 1; i < maxStreams; i++)
-           if (Stream[i])
-              {Stream[i]->isBound = false; Stream[i]->Stream[0] = 0;
-               if (Stream[i]->isDead) Stream[i]->Recycle(0, 0, 0);
-                  else Stream[i]->Link->Close();
-               Stream[i] = 0;
-              }
-       streamMutex.UnLock();
-      }
-
 // Handle statistics
 //
    SI->statsMutex.Lock();
@@ -962,11 +1100,6 @@ void XrdXrootdProtocol::Cleanup()
    if (AuthProt) {AuthProt->Delete(); AuthProt = 0;}
    if (Protect)  {Protect->Delete();  Protect  = 0;}
 
-// Handle parallel I/O appendages
-//
-   while((pioP = pioFirst)) {pioFirst = pioP->Next; pioP->Recycle();}
-   while((pioP = pioFree )) {pioFree  = pioP->Next; pioP->Recycle();}
-
 // Handle writev appendage
 //
    if (wvInfo) {free(wvInfo); wvInfo = 0;}
@@ -974,6 +1107,10 @@ void XrdXrootdProtocol::Cleanup()
 // Release aplication name
 //
    if (AppName) {free(AppName); AppName = 0;}
+
+// Release the pagewrite control object
+//
+   if (pgwCtl) delete pgwCtl;
 }
   
 /******************************************************************************/
@@ -1001,43 +1138,312 @@ int XrdXrootdProtocol::getData(const char *dtype, char *buff, int blen)
 }
 
 /******************************************************************************/
-/*                             g e t P a t h I D                              */
+  
+int XrdXrootdProtocol::getData(XrdXrootd::gdCallBack *cbP,
+                               const char *dtype, char *buff, int blen)
+{
+   bool inCB = (gdCtl.Status == GetDataCtl::inCallBk);
+
+// Setup the control information to direct the vector read
+//
+   memset((char *)&gdCtl, 0, sizeof(gdCtl));
+   gdCtl.BuffLen = blen;     // Buffer length (bytes to read)
+   gdCtl.Buffer  = buff;     // The actual buffer
+   gdCtl.CallBack= cbP;      // Method to callback upon success
+   gdCtl.ioDType = dtype;    // Name of the data being read for tracing
+   gdCtl.Status  = GetDataCtl::inData;
+
+// Effect the read. We prevent recursive calls if this was called while
+// we were in a callback, which is possible due to I/O continuations.
+//
+   if (inCB)
+      {gdCtl.useCB = true;
+       return 1;
+      }
+   return getDataCont();
+}
+
+/******************************************************************************/
+namespace {int consumed = 0;}
+  
+int XrdXrootdProtocol::getData(XrdXrootd::gdCallBack *cbP,
+                               const char *dtype, struct iovec *iov, int iovn)
+{
+   bool inCB = (gdCtl.Status == GetDataCtl::inCallBk);
+
+// Setup the control information to direct the vector read
+//
+   memset((char *)&gdCtl, 0, sizeof(gdCtl));
+   gdCtl.iovNum  = iovn;     // Number of original elements
+   gdCtl.iovVec  = iov;      // The actual vector
+   gdCtl.CallBack= cbP;      // Method to callback upon success
+   gdCtl.ioDType = dtype;    // Name of the data being read for tracing
+   gdCtl.Status  = GetDataCtl::inDataIov;
+
+// Effect the read. We prevent recursive calls if this was called while
+// we were in a callback, which is possible due to I/O continuations.
+//
+   if (inCB)
+      {gdCtl.useCB = true;
+       return 1;
+      }
+consumed = 0;
+   return getDataIovCont();
+}
+
+/******************************************************************************/
+/*                           g e t D a t a C o n t                            */
 /******************************************************************************/
 
-int XrdXrootdProtocol::getPathID(bool isRead)
+int XrdXrootdProtocol::getDataCont()
 {
-   XrdXrootdProtocol *pp;;
-   long long sBytes, bestBytes = 0;
-   int  bestStream = 0;
-   bool useS, sBusy, bestIsBusy = false;
+   int rlen;
 
-// Find a working stream that is least loaded and has at least one free
-// parallel I/O object (it might not be the fastest). We allow for a delay
-// of 50ms so as not to pick a busy stream.
+// Check if we need to terminate because the link died or we can proceed.
 //
-   for (int i = 1; i < maxStreams; i++)
-       {if (!(pp = Stream[i])) break;
-        pp->streamMutex.Lock();
-        if (!(pp->isDead || pp->isNOP))
-           {sBytes = (isRead ? pp->bytes2send : pp->bytes2recv);
-            sBusy  = (pp->pioFree == 0);
-                 if (!bestStream)        useS = true;
-            else if (bestBytes > sBytes) useS = !sBusy || bestIsBusy;
-            else if (!bestIsBusy)        useS = false;
-            else if (!sBusy)             useS = (bestBytes+5248000 >= sBytes);
-            else                         useS = false;
-            if (useS)
-               {bestBytes  = sBytes;
-                bestStream = i;
-                bestIsBusy = sBusy;
-               }
-           }
-        pp->streamMutex.UnLock();
-       }
+   if (gdCtl.linkWait.fetch_and(0) == GetDataCtl::Terminate)
+      return -EINPROGRESS;
 
-// All done, return result
+// I/O continuations may occur either via entry or an attempt to continue a new
+// operation via the callback. This takes care of it here.
 //
-   return bestStream;
+do{if ((rlen = Link->Recv(gdCtl.Buffer, gdCtl.BuffLen, readWait)) < 0) break;
+   gdCtl.Buffer  += rlen;
+   gdCtl.BuffLen -= rlen;
+
+// If we completed the read then either return or use the callback. Note that
+// we convert recursive call for more data into an iterative continuation.
+//
+   if (!gdCtl.BuffLen)
+      {if (gdCtl.useCB)
+          {gdCtl.Status = GetDataCtl::inCallBk;
+           if (int(gdCtl.stalls) > as_okstutter)
+              myStalls += int(gdCtl.stalls)/as_okstutter;
+           rlen = gdCtl.CallBack->gdDone();
+           if (rlen < 0) break;
+           if (gdCtl.Status == GetDataCtl::inData) continue;
+           if (gdCtl.Status == GetDataCtl::inDataIov) return getDataIovCont();
+           if (gdCtl.Status == GetDataCtl::inDump) return getDumpCont();
+           gdCtl.Status = GetDataCtl::inNone;
+           return rlen;
+          }
+       gdCtl.Status = GetDataCtl::inNone;
+       return 0;
+      }
+
+// Make sure we don't have an over-run
+//
+   if (gdCtl.BuffLen < 0)
+      {rlen = Link->setEtext("link excessive read length error");
+       break;
+      }
+
+// Record where we stopped and setup to resume here when more data arrives. We
+// must set myBlen to zero to avoid calling the other GetData() method. We want
+// to resume and perform the GetData() function here.
+//
+   Resume = &XrdXrootdProtocol::getDataCont;
+   myBlen = 0;
+   gdCtl.useCB    = true;
+   gdCtl.linkWait = GetDataCtl::Active;
+   if (gdCtl.stalls < 255) gdCtl.stalls++;
+
+// Return indicating we need more data
+//
+   TRACEP(REQ, gdCtl.ioDType <<" timeout; read " <<rlen <<" bytes "
+                             <<gdCtl.BuffLen <<" remaining");
+   return 1;
+
+  } while(true);
+
+// If got here then we had a link failure or some other fatal issue
+//
+   if (rlen != -ENOMSG) return Link->setEtext("link read error");
+
+// Use callback, if need be.
+//
+   if (gdCtl.useCB)
+      {gdCtl.Status = GetDataCtl::inCallBk;
+       gdCtl.CallBack->gdFail();
+      }
+   gdCtl.Status = GetDataCtl::inNone;
+   return -1;
+}
+  
+/******************************************************************************/
+/*                         g e t D a t I o v C o n t                          */
+/******************************************************************************/
+
+int XrdXrootdProtocol::getDataIovCont()
+{
+   int rc;
+
+// Check if we need to terminate because the link died or we can proceed.
+//
+   if (gdCtl.linkWait.fetch_and(0) == GetDataCtl::Terminate)
+      return -EINPROGRESS;
+
+// I/O continuations may occur either via entry or an attempt to continue a new
+// operation via the callback. This takes care of it here.
+//
+do{struct iovec *ioV = gdCtl.iovVec;
+   int i, rlen, iovN = gdCtl.iovNum, iNow = gdCtl.iovNow;
+
+// Read as much data as we can. Handle any link error. Note that when a link
+// error occurs we return failure whether or not the callback wants to do more.
+//
+   rlen = Link->Recv(&ioV[iNow], iovN - iNow, readWait);
+   if (rlen  < 0)
+      {if (rlen != -ENOMSG) Link->setEtext("link read error");
+       if (gdCtl.iovAdj)
+          {ioV[iNow].iov_base = ((char *)ioV[iNow].iov_base) - gdCtl.iovAdj;
+           ioV[iNow].iov_len += gdCtl.iovAdj;
+           gdCtl.iovAdj = 0;
+          }
+       rc = -1;
+       break;
+      }
+
+// Compute where we finished in the iovec.
+//
+   for (i = iNow; i < iovN && (int)ioV[i].iov_len <= rlen; i++)
+       rlen -= ioV[i].iov_len;
+
+// Before proceeding, restore any changes we made to a completed iovec element
+//
+   if (i != iNow && gdCtl.iovAdj)
+      {ioV[iNow].iov_base = ((char *)ioV[iNow].iov_base) - gdCtl.iovAdj;
+       ioV[iNow].iov_len += gdCtl.iovAdj;
+       gdCtl.iovAdj = 0;
+      }
+
+// If the vector is complete then effect the callback unless this was the
+// initial call, we simply return to prevent recursive continuations by
+// converting a recursive call to an iterative continuation!
+//
+   if (i >= iovN)
+      {if (!rlen) 
+          {if (gdCtl.useCB)
+              {gdCtl.Status = GetDataCtl::inCallBk;
+               if (int(gdCtl.stalls) > as_okstutter)
+                  myStalls += int(gdCtl.stalls)/as_okstutter;
+               rc = gdCtl.CallBack->gdDone();
+               if (rc < 0) break;
+               if (gdCtl.Status == GetDataCtl::inDataIov) continue;
+               if (gdCtl.Status == GetDataCtl::inDump) return getDumpCont();
+               gdCtl.Status = GetDataCtl::inNone;
+               return rc;
+              }
+           gdCtl.Status = GetDataCtl::inNone;
+           return 0;
+          }
+       rc = Link->setEtext("link iov read length error");
+       break;
+      }
+
+// Record where we stopped and adjust the iovec element address and length if
+// needed. Record the change made so that it can be undone as we progress.
+//
+   gdCtl.iovNow = i;
+   if (rlen)
+      {if (gdCtl.iovAdj == 0) gdCtl.iovNow = i;
+       gdCtl.iovAdj += rlen;
+       ioV[i].iov_base = ((char *)ioV[i].iov_base) + rlen;
+       ioV[i].iov_len -= rlen;
+      }
+
+// Setup to resume here when more data arrives. We must set myBlen to zero to
+// avoid calling the other GetData() method as we want to resume here.
+//
+   Resume = &XrdXrootdProtocol::getDataIovCont;
+   myBlen = 0;
+   gdCtl.useCB    = true;
+   gdCtl.linkWait = GetDataCtl::Active;
+   if (gdCtl.stalls < 255) gdCtl.stalls++;
+
+// Return indicating we need more data
+//
+   TRACEP(REQ, gdCtl.ioDType<<" read timeout; "<<iovN-i<<" of "
+             <<gdCtl.iovNum <<" iov elements left");
+   return 1;
+
+  } while(true);
+
+// If got here then we had a link failure or some other fatal issue
+//
+   if (gdCtl.useCB)
+      {gdCtl.Status = GetDataCtl::inCallBk;
+       gdCtl.CallBack->gdFail();
+      }
+   gdCtl.Status = GetDataCtl::inNone;
+   return rc;
+}
+
+/******************************************************************************/
+/*                               g e t D u m p                                */
+/******************************************************************************/
+  
+int XrdXrootdProtocol::getDump(const char *dtype, int dlen)
+{
+   bool inCB = (gdCtl.Status == GetDataCtl::inCallBk);
+
+// Setup the control information to direct the vector read
+//
+   memset((char *)&gdCtl, 0, sizeof(gdCtl));
+   gdCtl.DumpLen = dlen;     // Bytes left to drain
+   gdCtl.ioDType = dtype;    // Name of the data being read for tracing
+   gdCtl.Status  = GetDataCtl::inDump;
+
+// Effect the read. We prevent recursive calls if this was called while
+// we were in a callback, which is possible due to I/O continuations.
+//
+   return (inCB ? 1 : getDumpCont());
+}
+
+/******************************************************************************/
+/* Private:                  g e t D u m p C o n t                            */
+/******************************************************************************/
+
+int XrdXrootdProtocol::getDumpCont()
+{
+    int  rlen = 0, rwant;
+    char buff[65536];
+
+   TRACEP(REQ, gdCtl.ioDType<<" discarding "<<gdCtl.DumpLen<<" bytes.");
+
+// Read data and discard it
+//
+   while(gdCtl.DumpLen > 0)
+        {if (gdCtl.DumpLen <= (int)sizeof(buff)) rwant = gdCtl.DumpLen;
+            else rwant = sizeof(buff);
+         if ((rlen = Link->Recv(buff, rwant, readWait)) <= 0) break;
+         gdCtl.DumpLen -= rlen;
+        }
+
+// Check if we failed
+//
+   if (rlen < 0 || gdCtl.DumpLen < 0)
+      {if (gdCtl.DumpLen < 0) Link->setEtext("link read overrun error");
+          else if (rlen != -ENOMSG) Link->setEtext("link read error");
+       gdCtl.Status = GetDataCtl::inNone;
+       return -1;
+      }
+
+// Check if we completed
+//
+   if (gdCtl.DumpLen == 0)
+      {gdCtl.Status = GetDataCtl::inNone;
+       return 0;
+      }
+
+// Wait until more data arrives. We will now need to use the callback.
+//
+   Resume = &XrdXrootdProtocol::getDumpCont;
+   myBlen = 0;
+
+   TRACEP(REQ, gdCtl.ioDType<<" read timeout; "<<gdCtl.DumpLen
+             <<" bytes left to discard");
+   return 1;
 }
   
 /******************************************************************************/
@@ -1050,16 +1456,15 @@ void XrdXrootdProtocol::Reset()
    argp               = 0;
    Link               = 0;
    FTab               = 0;
+   pmHandle           = 0;
+   ResumePio          = 0;
    Resume             = 0;
    myBuff             = (char *)&Request;
    myBlen             = sizeof(Request);
    myBlast            = 0;
-   myOffset           = 0;
-   myIOLen            = 0;
    myStalls           = 0;
-   myFlags            = 0;
-   myAioReq           = 0;
-   myFile             = 0;
+   pgwCtl             = 0;
+   memset(&IO, 0, sizeof(IO));
    wvInfo             = 0;
    numReads           = 0;
    numReadP           = 0;
@@ -1086,21 +1491,25 @@ void XrdXrootdProtocol::Reset()
    CapVer             = 0;
    clientPV           = 0;
    clientRN           = 0;
+   pmDone             = false;
    reTry              = 0;
+   endNote            = 0;
    PathID             = 0;
+   newPio             = false;
    rvSeq              = 0;
    wvSeq              = 0;
    doTLS              = tlsNot; // Assume client is not capable. This will be
    ableTLS            = false;  // resolved during the kXR_protocol interchange.
    isTLS              = false;  // Made true when link converted to TLS
+   linkAioReq         = {0};
    pioFree = pioFirst = pioLast = 0;
-   bytes2recv = bytes2send = 0;
-   isActive = isDead  = isNOP = isBound = false;
+   isActive = isLinkWT= isNOP = isDead = false;
    sigNeed = sigHere = sigRead = false;
    sigWarn = true;
    rdType             = 0;
    Entity.Reset(0);
    memset(Stream,  0, sizeof(Stream));
+   memset((char *)&gdCtl,  0, sizeof(gdCtl));
    PrepareCount       = 0;
    if (AppName) {free(AppName); AppName = 0;}
 }
