@@ -28,7 +28,7 @@
 /* specific prior written permission of the institution or contributor.       */
 /******************************************************************************/
 
-#include <string.h>
+#include <cstring>
 #include <unistd.h>
 #include <sys/uio.h>
 
@@ -52,7 +52,7 @@
 /*                        C l o b a l   S y m b o l s                         */
 /******************************************************************************/
   
-extern  XrdOucTrace *XrdXrootdTrace;
+extern  XrdSysTrace  XrdXrootdTrace;
 
 #undef  TRACELINK
 #define TRACELINK Link
@@ -134,7 +134,7 @@ int XrdXrootdTransit::AttnCont(XrdXrootdTransPend *tP,  int rcode,
        return 0;
       }
 
-// Send off the defered response
+// Send off the deferred response
 //
    rc = Send(rcode, ioV, ioN, ioL);
 
@@ -215,7 +215,7 @@ int XrdXrootdTransit::Fatal(int rc)
 
 void XrdXrootdTransit::Init(XrdScheduler *schedP, int qMax, int qTTL)
 {
-   TranStack.Set(schedP, XrdXrootdTrace, TRACE_MEM);
+   TranStack.Set(schedP, &XrdXrootdTrace, TRACE_MEM);
    TranStack.Set(qMax, qTTL);
 }
 
