@@ -132,11 +132,10 @@ void FileCopyTest::DownloadTestFunc()
   std::string transferSum = "zcrc32:"; transferSum += crcBuff;
 
   std::string remoteSum;
-  std::string dataServer;
-  CPPUNIT_ASSERT( f.GetProperty( "DataServer", dataServer ) );
+  std::string lastUrl;
+  CPPUNIT_ASSERT( f.GetProperty( "LastURL", lastUrl ) );
   CPPUNIT_ASSERT_XRDST( Utils::GetRemoteCheckSum( remoteSum, "zcrc32",
-                                                  dataServer,
-                                                  remoteFile ) );
+                                                  URL( lastUrl ) ) );
   CPPUNIT_ASSERT( remoteSum == transferSum );
 
   delete stat;
@@ -230,10 +229,10 @@ void FileCopyTest::UploadTestFunc()
   XrdCksData crc; crc.Set( (const void *)crc32Sum->Final(), 4 ); crc.Get( crcBuff, 9 );
   std::string transferSum = "zcrc32:"; transferSum += crcBuff;
 
-  std::string remoteSum, dataServer;
-  f.GetProperty( "DataServer", dataServer );
+  std::string remoteSum, lastUrl;
+  f.GetProperty( "LastURL", lastUrl );
   CPPUNIT_ASSERT_XRDST( Utils::GetRemoteCheckSum( remoteSum, "zcrc32",
-                                                  dataServer, remoteFile ) );
+                                                  lastUrl ) );
   CPPUNIT_ASSERT( remoteSum == transferSum );
 
   //----------------------------------------------------------------------------
@@ -347,7 +346,7 @@ void FileCopyTest::CopyTestFunc( bool thirdParty )
   std::string localFile    = "/data/localfile.dat";
 
   CopyProcess  process1, process2, process3, process4, process5, process6, process7, process8, process9,
-               process10, process11, process12, process13;
+               process10, process11, process12, process13, process14, process15, process16, process17;
   PropertyList properties, results;
   FileSystem fs( manager2 );
 
@@ -416,12 +415,78 @@ void FileCopyTest::CopyTestFunc( bool thirdParty )
     CPPUNIT_ASSERT_XRDST_NOTOK( process13.Run(0), XrdCl::errCheckSumError );
     env->PutInt( "ZipMtlnCksum", 0 );
     CPPUNIT_ASSERT_XRDST( fs.Rm( targetPath ) );
+
+    //--------------------------------------------------------------------------
+    // Copy with
+    //   `--xrate`
+    //   `--xrate-threshold`
+    //--------------------------------------------------------------------------
+    results.Clear();
+    properties.Clear();
+    properties.Set( "source",          sourceURL        );
+    properties.Set( "target",          targetURL        );
+    properties.Set( "xrate",           1024 * 1024 * 32 ); //< limit the transfer rate to 32MB/s
+    properties.Set( "xrateThreshold", 1024 * 1024 * 30 ); //< fail the job if the transfer rate drops under 30MB/s
+    CPPUNIT_ASSERT_XRDST( process14.AddJob( properties, &results ) );
+    CPPUNIT_ASSERT_XRDST( process14.Prepare() );
+    CPPUNIT_ASSERT_XRDST( process14.Run(0) );
+    CPPUNIT_ASSERT_XRDST( fs.Rm( targetPath ) );
+
+    //--------------------------------------------------------------------------
+    // Now test the cp-timeout
+    //--------------------------------------------------------------------------
+    results.Clear();
+    properties.Clear();
+    properties.Set( "source",    sourceURL   );
+    properties.Set( "target",    targetURL   );
+    properties.Set( "xrate",     1024 * 1024 ); //< limit the transfer rate to 1MB/s (the file is 1GB big so the transfer will take 1024 seconds)
+    properties.Set( "cpTimeout", 10          ); //< timeout the job after 10 seconds
+    CPPUNIT_ASSERT_XRDST( process15.AddJob( properties, &results ) );
+    CPPUNIT_ASSERT_XRDST( process15.Prepare() );
+    CPPUNIT_ASSERT_XRDST_NOTOK( process15.Run(0), XrdCl::errOperationExpired );
+    CPPUNIT_ASSERT_XRDST( fs.Rm( targetPath ) );
+
+    //--------------------------------------------------------------------------
+    // Test posc for local files
+    //--------------------------------------------------------------------------
+    results.Clear();
+    properties.Clear();
+    std::string localtrg = "file://localhost/data/tpcFile.dat";
+    properties.Set( "source",    sourceURL );
+    properties.Set( "target",    localtrg  );
+    properties.Set( "posc",      true      );
+    CancelProgressHandler progress16; //> abort the copy after 100MB
+    CPPUNIT_ASSERT_XRDST( process16.AddJob( properties, &results ) );
+    CPPUNIT_ASSERT_XRDST( process16.Prepare() );
+    CPPUNIT_ASSERT_XRDST_NOTOK( process16.Run( &progress16 ), errOperationInterrupted );
+    XrdCl::FileSystem localfs( "file://localhost" );
+    XrdCl::StatInfo *ptr = nullptr;
+    CPPUNIT_ASSERT_XRDST_NOTOK( localfs.Stat( "/data/tpcFile.dat", ptr ), XrdCl::errLocalError );
+
+    //--------------------------------------------------------------------------
+    // Test --retry and --retry-policy
+    //--------------------------------------------------------------------------
+    results.Clear();
+    properties.Clear();
+    properties.Set( "xrate",     1024 * 1024 * 32 ); //< limit the transfer rate to 32MB/s
+    properties.Set( "cpTimeout", 20               ); //< timeout the job after 20 seconds
+    properties.Set( "source",    sourceURL        );
+    properties.Set( "target",    targetURL        );
+    env->PutInt( "CpRetry", 1 );
+    env->PutString( "CpRetryPolicy", "continue" );
+    CPPUNIT_ASSERT_XRDST( process17.AddJob( properties, &results ) );
+    CPPUNIT_ASSERT_XRDST( process17.Prepare() );
+    CPPUNIT_ASSERT_XRDST( process17.Run(0) );
+    CPPUNIT_ASSERT_XRDST( fs.Rm( targetPath ) );
+    env->PutInt( "CpRetry", XrdCl::DefaultCpRetry );
+    env->PutString( "CpRetryPolicy", XrdCl::DefaultCpRetryPolicy );
   }
 
   //----------------------------------------------------------------------------
   // Copy from a Metalink
   //----------------------------------------------------------------------------
   results.Clear();
+  properties.Clear();
   properties.Set( "source",       metalinkURL );
   properties.Set( "target",       targetURL   );
   properties.Set( "checkSumMode", "end2end"   );
