@@ -31,6 +31,7 @@
 #include "XrdCl/XrdClParallelOperation.hh"
 #include "XrdCl/XrdClFileOperations.hh"
 #include "XrdCl/XrdClFileSystemOperations.hh"
+#include "XrdCl/XrdClCheckpointOperation.hh"
 #include "XrdCl/XrdClFwd.hh"
 
 #include <algorithm>
@@ -55,6 +56,7 @@ class WorkflowTest: public CppUnit::TestCase
       CPPUNIT_TEST( WorkflowWithFutureTest );
       CPPUNIT_TEST( XAttrWorkflowTest );
       CPPUNIT_TEST( MkDirAsyncTest );
+      CPPUNIT_TEST( CheckpointTest );
     CPPUNIT_TEST_SUITE_END();
     void ReadingWorkflowTest();
     void WritingWorkflowTest();
@@ -67,6 +69,7 @@ class WorkflowTest: public CppUnit::TestCase
     void WorkflowWithFutureTest();
     void XAttrWorkflowTest();
     void MkDirAsyncTest();
+    void CheckpointTest();
 };
 
 CPPUNIT_TEST_SUITE_REGISTRATION( WorkflowTest );
@@ -198,7 +201,7 @@ void WorkflowTest::ReadingWorkflowTest(){
     uint64_t offset = 0;
 
     auto &&pipe = Open( f, fileUrl, flags ) >> openHandler // by reference
-                | Stat( f, true) >> [size, buffer]( XRootDStatus &status, StatInfo &stat )
+                | Stat( f, true) >> [size, buffer]( XRootDStatus &status, StatInfo &stat ) mutable
                     {
                       CPPUNIT_ASSERT_XRDST( status );
                       CPPUNIT_ASSERT( stat.GetSize() == 1048576000 );
@@ -254,30 +257,28 @@ void WorkflowTest::WritingWorkflowTest(){
     //----------------------------------------------------------------------------
     // Forward parameters between operations
     //----------------------------------------------------------------------------
-    Fwd<iovec*>   iov;
-    Fwd<int>      iovcnt;
-    Fwd<uint32_t> size;
-    Fwd<void*>    buffer;
+    Fwd<std::vector<iovec>> iov;
+    Fwd<uint32_t>           size;
+    Fwd<void*>              buffer;
 
     //----------------------------------------------------------------------------
     // Create and execute workflow
     //----------------------------------------------------------------------------
-    Pipeline pipe = Open( f, fileUrl, flags ) >> [iov, iovcnt, texts]( XRootDStatus &status )
+    Pipeline pipe = Open( f, fileUrl, flags ) >> [iov, texts]( XRootDStatus &status ) mutable
                       {
                         CPPUNIT_ASSERT_XRDST( status );
-                        iovec *vec = new iovec[3];
+                        std::vector<iovec> vec( 3 );
                         vec[0].iov_base = strdup( texts[0].c_str() );
                         vec[0].iov_len  = texts[0].size();
                         vec[1].iov_base = strdup( texts[1].c_str() );
                         vec[1].iov_len  = texts[1].size();
                         vec[2].iov_base = strdup( texts[2].c_str() );
                         vec[2].iov_len  = texts[2].size();
-                        iov = vec;
-                        iovcnt = 3;
+                        iov = std::move( vec );
                       }
-                  | WriteV( f, 0, iov, iovcnt )
+                  | WriteV( f, 0, iov )
                   | Sync( f )
-                  | Stat( f, true ) >> [size, buffer, createdFileSize]( XRootDStatus &status, StatInfo &info )
+                  | Stat( f, true ) >> [size, buffer, createdFileSize]( XRootDStatus &status, StatInfo &info ) mutable
                       {
                         CPPUNIT_ASSERT_XRDST( status );
                         CPPUNIT_ASSERT( createdFileSize == info.GetSize() );
@@ -292,11 +293,9 @@ void WorkflowTest::WritingWorkflowTest(){
     CPPUNIT_ASSERT_XRDST( status );
     CPPUNIT_ASSERT( rdresp.get() == texts[0] + texts[1] + texts[2] );
 
-    iovec *vec = *iov;
-    free( vec[0].iov_base );
-    free( vec[1].iov_base );
-    free( vec[2].iov_base );
-    delete[] vec;
+    free( (*iov)[0].iov_base );
+    free( (*iov)[1].iov_base );
+    free( (*iov)[2].iov_base );
 }
 
 
@@ -324,7 +323,7 @@ void WorkflowTest::MissingParameterTest(){
     // Create and execute workflow
     //----------------------------------------------------------------------------
 
-    bool pipebroken = false;
+    bool error = false, closed = false;
     const OpenFlags::Flags flags = OpenFlags::Read;
     uint64_t offset = 0;
 
@@ -333,8 +332,12 @@ void WorkflowTest::MissingParameterTest(){
                   | Read( f, offset, size, buffer ) >> readHandler // by reference
                   | Close( f ) >> [&]( XRootDStatus& st )
                       {
-                        pipebroken = !st.IsOK() && ( st.code == errPipelineFailed );
-                      };
+                        closed = true;
+                      }
+                  | Final( [&]( const XRootDStatus& st )
+                      {
+                        error = !st.IsOK();
+                      });
 
     XRootDStatus status = WaitFor( std::move( pipe ) );
     CPPUNIT_ASSERT( status.IsError() );
@@ -342,7 +345,7 @@ void WorkflowTest::MissingParameterTest(){
     // If there is an error, last handlers should not be executed
     //----------------------------------------------------------------------------
     CPPUNIT_ASSERT( readHandler.Executed() );
-    CPPUNIT_ASSERT( pipebroken );
+    CPPUNIT_ASSERT( !closed & error );
 }
 
 
@@ -514,7 +517,7 @@ void WorkflowTest::ParallelTest(){
     Fwd<std::string> url1, url2;
 
     bool lockHandlerExecuted = false;
-    auto lockOpenHandler = [&,url1, url2]( XRootDStatus &status )
+    auto lockOpenHandler = [&,url1, url2]( XRootDStatus &status ) mutable
         {
           url1 = GetFileUrl( "cb4aacf1-6f28-42f2-b68a-90a73460f424.dat" );
           url2 = GetFileUrl( dataFileName );
@@ -575,6 +578,31 @@ void WorkflowTest::ParallelTest(){
 
     delete f;
     delete dataF;
+
+    //----------------------------------------------------------------------------
+    // Test the policies
+    //----------------------------------------------------------------------------
+    std::string url_exists = "/data/1db882c8-8cd6-4df1-941f-ce669bad3458.dat";
+    std::string not_exists = "/data/blablabla.txt";
+    CPPUNIT_ASSERT_XRDST( WaitFor( Parallel( Stat( fs, url_exists ), Stat( fs, url_exists ) ).Any() ) );
+
+    std::string also_exists = "/data/3c9a9dd8-bc75-422c-b12c-f00604486cc1.dat";
+    CPPUNIT_ASSERT_XRDST( WaitFor( Parallel( Stat( fs, url_exists ),
+                                             Stat( fs, also_exists ),
+                                             Stat( fs, not_exists ) ).Some( 2 ) ) );
+    std::atomic<int> errcnt( 0 );
+    std::atomic<int> okcnt( 0 );
+
+    auto hndl = [&]( auto s, auto i )
+      {
+        if( s.IsOK() ) ++okcnt;
+        else ++errcnt;
+      };
+
+    CPPUNIT_ASSERT_XRDST( WaitFor( Parallel( Stat( fs, url_exists )  >> hndl,
+                                             Stat( fs, also_exists ) >> hndl,
+                                             Stat( fs, not_exists )  >> hndl ).AtLeast( 1 ) ) );
+    CPPUNIT_ASSERT( okcnt == 2 && errcnt == 1 );
 }
 
 
@@ -663,7 +691,7 @@ void WorkflowTest::MixedWorkflowTest(){
       auto &&operation = Open( file[i], url[i], flags )
                        | Write( file[i], 0, length[i], text[i] )
                        | Sync( file[i] )
-                       | Stat( file[i], true ) >> [size, buffer, i]( XRootDStatus &status, StatInfo &info )
+                       | Stat( file[i], true ) >> [size, buffer, i]( XRootDStatus &status, StatInfo &info ) mutable
                            {
                              CPPUNIT_ASSERT_XRDST( status );
                              size[i] = info.GetSize();
@@ -901,3 +929,63 @@ void WorkflowTest::MkDirAsyncTest() {
   CPPUNIT_ASSERT(t.get().status == stOK); 
 }
 
+void WorkflowTest::CheckpointTest() {
+  using namespace XrdCl;
+
+  File f1;
+  const char data[] = "Murzynek Bambo w Afryce mieszka,\n"
+                      "czarna ma skore ten nasz kolezka\n"
+                      "Uczy sie pilnie przez cale ranki\n"
+                      "Ze swej murzynskiej pierwszej czytanki.";
+  std::string url = "root://localhost//data/chkpttest.txt";
+
+  CPPUNIT_ASSERT_XRDST( WaitFor( Open( f1, url, OpenFlags::New | OpenFlags::Write ) |
+                                 Write( f1, 0, sizeof( data ), data ) |
+                                 Close( f1 ) ) );
+
+  //---------------------------------------------------------------------------
+  // Update the file without commiting the checkpoint
+  //---------------------------------------------------------------------------
+  File f2;
+  const char update[] = "Jan A Kowalski";
+
+  CPPUNIT_ASSERT_XRDST( WaitFor( Open( f2, url, OpenFlags::Update ) |
+                                 Checkpoint( f2, ChkPtCode::BEGIN ) |
+                                 ChkptWrt( f2, 0, sizeof( update ), update ) |
+                                 Close( f2 ) ) );
+
+  File f3;
+  char readout[sizeof( data )];
+  // readout the data to see if the update was succesful (it shouldn't be)
+  CPPUNIT_ASSERT_XRDST( WaitFor( Open( f3, url, OpenFlags::Read ) |
+                                 Read( f3, 0, sizeof( readout ), readout ) |
+                                 Close( f3 ) ) );
+  // we expect the data to be unchanged
+  CPPUNIT_ASSERT( strncmp( readout, data, sizeof( data ) ) == 0 );
+
+  //---------------------------------------------------------------------------
+  // Update the file and commit the changes
+  //---------------------------------------------------------------------------
+  File f4;
+  CPPUNIT_ASSERT_XRDST( WaitFor( Open( f4, url, OpenFlags::Update ) |
+                                 Checkpoint( f4, ChkPtCode::BEGIN ) |
+                                 ChkptWrt( f4, 0, sizeof( update ), update ) |
+                                 Checkpoint( f4, ChkPtCode::COMMIT ) |
+                                 Close( f4 ) ) );
+  File f5;
+  // readout the data to see if the update was succesful (it shouldn't be)
+  CPPUNIT_ASSERT_XRDST( WaitFor( Open( f5, url, OpenFlags::Read ) |
+                                 Read( f5, 0, sizeof( readout ), readout ) |
+                                 Close( f5 ) ) );
+  // we expect the data to be unchanged
+  CPPUNIT_ASSERT( strncmp( readout, update, sizeof( update ) ) == 0 );
+  CPPUNIT_ASSERT( strncmp( readout + sizeof( update ), data + sizeof( update ),
+                           sizeof( data ) - sizeof( update ) ) == 0 );
+
+  //---------------------------------------------------------------------------
+  // Now clean up
+  //---------------------------------------------------------------------------
+  FileSystem fs( url );
+  CPPUNIT_ASSERT_XRDST( WaitFor( Rm( fs, "/data/chkpttest.txt" ) ) );
+}
+
