@@ -199,9 +199,9 @@ void UtilsTest::URLTest()
   CPPUNIT_ASSERT( url8.GetProtocol() == "file" );
   CPPUNIT_ASSERT( url8.GetUserName() == "" );
   CPPUNIT_ASSERT( url8.GetPassword() == "" );
-  CPPUNIT_ASSERT( url8.GetHostName() == "" );
+  CPPUNIT_ASSERT( url8.GetHostName() == "localhost" );
   CPPUNIT_ASSERT( url8.GetPath() == "/etc/passwd" );
-
+  CPPUNIT_ASSERT( url8.GetHostId() == "localhost" );
   CPPUNIT_ASSERT( url8.GetPathWithParams() == "/etc/passwd" );
   CPPUNIT_ASSERT( url8.GetParams().size() == 0 );
 
@@ -378,7 +378,7 @@ void UtilsTest::TaskManagerTest()
 void UtilsTest::SIDManagerTest()
 {
   using namespace XrdCl;
-  SIDManager manager;
+  std::shared_ptr<SIDManager> manager = SIDMgrPool::Instance().GetSIDMgr( "root://fake:1094//dir/file" );
 
   uint8_t sid1[2];
   uint8_t sid2[2];
@@ -386,26 +386,26 @@ void UtilsTest::SIDManagerTest()
   uint8_t sid4[2];
   uint8_t sid5[2];
 
-  CPPUNIT_ASSERT_XRDST( manager.AllocateSID( sid1 ) );
-  CPPUNIT_ASSERT_XRDST( manager.AllocateSID( sid2 ) );
-  manager.ReleaseSID( sid2 );
-  CPPUNIT_ASSERT_XRDST( manager.AllocateSID( sid3 ) );
-  CPPUNIT_ASSERT_XRDST( manager.AllocateSID( sid4 ) );
-  CPPUNIT_ASSERT_XRDST( manager.AllocateSID( sid5 ) );
+  CPPUNIT_ASSERT_XRDST( manager->AllocateSID( sid1 ) );
+  CPPUNIT_ASSERT_XRDST( manager->AllocateSID( sid2 ) );
+  manager->ReleaseSID( sid2 );
+  CPPUNIT_ASSERT_XRDST( manager->AllocateSID( sid3 ) );
+  CPPUNIT_ASSERT_XRDST( manager->AllocateSID( sid4 ) );
+  CPPUNIT_ASSERT_XRDST( manager->AllocateSID( sid5 ) );
 
   CPPUNIT_ASSERT( (sid1[0] != sid2[0]) || (sid1[1] != sid2[1]) );
-  CPPUNIT_ASSERT( manager.NumberOfTimedOutSIDs() == 0 );
-  manager.TimeOutSID( sid4 );
-  manager.TimeOutSID( sid5 );
-  CPPUNIT_ASSERT( manager.NumberOfTimedOutSIDs() == 2 );
-  CPPUNIT_ASSERT( manager.IsTimedOut( sid3 ) == false );
-  CPPUNIT_ASSERT( manager.IsTimedOut( sid1 ) == false );
-  CPPUNIT_ASSERT( manager.IsTimedOut( sid4 ) == true );
-  CPPUNIT_ASSERT( manager.IsTimedOut( sid5 ) == true );
-  manager.ReleaseTimedOut( sid5 );
-  CPPUNIT_ASSERT( manager.IsTimedOut( sid5 ) == false );
-  manager.ReleaseAllTimedOut();
-  CPPUNIT_ASSERT( manager.NumberOfTimedOutSIDs() == 0 );
+  CPPUNIT_ASSERT( manager->NumberOfTimedOutSIDs() == 0 );
+  manager->TimeOutSID( sid4 );
+  manager->TimeOutSID( sid5 );
+  CPPUNIT_ASSERT( manager->NumberOfTimedOutSIDs() == 2 );
+  CPPUNIT_ASSERT( manager->IsTimedOut( sid3 ) == false );
+  CPPUNIT_ASSERT( manager->IsTimedOut( sid1 ) == false );
+  CPPUNIT_ASSERT( manager->IsTimedOut( sid4 ) == true );
+  CPPUNIT_ASSERT( manager->IsTimedOut( sid5 ) == true );
+  manager->ReleaseTimedOut( sid5 );
+  CPPUNIT_ASSERT( manager->IsTimedOut( sid5 ) == false );
+  manager->ReleaseAllTimedOut();
+  CPPUNIT_ASSERT( manager->NumberOfTimedOutSIDs() == 0 );
 }
 
 //------------------------------------------------------------------------------
